"""DDans library"""

__version__ = "0.1.10"

from . import common
from . import config
from . import data
from . import dbase
from . import descriptor
from . import domain
from . import module
from . import native
from . import single

from . import dfunc
from . import pbar
from . import terminal
from . import utils

__all__ = [
    "common",
    "config",
    "data",
    "dbase",
    "descriptor",
    "domain",
    "module",
    "native",
    "single",
    "dfunc",
    "pbar",
    "terminal",
    "utils",
]



"""Module utilities"""

from . import aes
from . import args_parser
from . import cmd
from . import gi18n
from . import icon
from . import queue
from . import sqlite
from . import stack
from . import xlsx

__all__ = [
    "aes",
    "args_parser",
    "cmd",
    "gi18n",
    "icon",
    "queue",
    "sqlite",
    "stack",
    'xlsx'
]



# -*- coding: utf-8 -*-
from typing import List, Generic
from ddans.common.type import T


class Stack(Generic[T]):

    def __init__(self):
        """
        初始化一个空栈。
        """
        self.stack: List[T] = []

    def push(self, item: T) -> None:
        """
        将一个元素压入栈中。

        :param item: 要压入栈的元素
        """
        self.stack.append(item)

    def pop(self) -> T:
        """
        移除并返回栈顶元素。
        如果栈为空，则抛出 IndexError。

        :return: 栈顶元素
        :raises IndexError: 如果栈为空
        """
        if not self.is_empty():
            return self.stack.pop()
        else:
            raise IndexError("从空栈中弹出元素")

    def peek(self) -> T:
        """
        返回栈顶元素但不移除它。
        如果栈为空，则抛出 IndexError。

        :return: 栈顶元素
        :raises IndexError: 如果栈为空
        """
        if not self.is_empty():
            return self.stack[-1]
        else:
            raise IndexError("从空栈中窥视元素")

    def is_empty(self) -> bool:
        """
        检查栈是否为空。

        :return: 如果栈为空则返回 True，否则返回 False
        """
        return len(self.stack) == 0

    def size(self) -> int:
        """
        返回栈中元素的数量。

        :return: 栈中元素的数量
        """
        return len(self.stack)

    def clear(self) -> None:
        """
        移除栈中所有元素。
        """
        self.stack.clear()

    def __repr__(self) -> str:
        """
        返回栈的字符串表示。

        :return: 栈的字符串表示
        """
        return f"Stack({self.stack})"



# -*- coding: utf-8 -*-

from ddans.native.system import NSystem


class NConfig():

    @staticmethod
    def app_data(path: str = ""):
        appData = NSystem.get_app_data()
        return NSystem.join(appData, 'ddan-py', path)



# -*- coding: utf-8 -*-
from datetime import datetime
import sqlite3

from ddans.native.system import NSystem
from ddans.native.config import NConfig
from ddans.native.hook import NHook
from ddans.native.log import NLog


class DBModel:
    _fields = {}

    @classmethod
    def create_table(cls):
        dbfilename = cls.dbfilename()
        NSystem.ensure_dir(dbfilename)

        existing_columns = cls.table_columns()
        if not existing_columns:
            cls._create_table()
            return

        # Add missing columns
        for name, field in cls._fields.items():
            if name not in existing_columns:
                sql = (f"ALTER TABLE {cls.__name__.lower()} ADD COLUMN "
                       f"{name} {field.field_type}")
                if field.default is not None:
                    sql += f" DEFAULT {field.default}"
                cls._execute(sql)

    @classmethod
    def _create_table(cls):
        fields = []
        for name, field in cls._fields.items():
            field_def = f"{name} {field.field_type}"
            if field.primary_key:
                field_def += " PRIMARY KEY"
                if field.autoincrement:
                    field_def += " AUTOINCREMENT"
            if not field.nullable:
                field_def += " NOT NULL"
            if field.default is not None:
                field_def += f" DEFAULT {field.default}"
            fields.append(field_def)
        sql = (f"CREATE TABLE IF NOT EXISTS {cls.__name__.lower()} "
               f"({', '.join(fields)})")
        NSystem.ensure_dir(cls.dbfilename())
        cls._execute(sql)

    @classmethod
    def table_columns(cls):
        sql = f"PRAGMA table_info({cls.__name__.lower()})"
        rows = cls._execute(sql, callback=lambda cur: cur.fetchall())
        if rows:
            columns = [column[1] for column in rows]
            return columns
        else:
            return None

    @classmethod
    def dbfilename(cls, name="ddan"):
        return NSystem.join(NConfig.app_data("base"), f"{name}.db")

    @classmethod
    def sqlSelect(cls, select="*", where=None):
        where
        if NHook.isvalid_str(where):
            sql = f"SELECT {select} FROM {cls.__name__.lower()} WHERE {where}"
        else:
            sql = f"SELECT {select} FROM {cls.__name__.lower()}"
        return sql

    @classmethod
    def _execute(cls, sql: str, params=None, callback=None):
        result = None
        if not NHook.isvalid_str(sql):
            return result

        with sqlite3.connect(cls.dbfilename()) as conn:
            cur = conn.cursor()
            try:
                if params:
                    cur.execute(sql, params)
                else:
                    cur.execute(sql)
                if callback:
                    result = callback(cur)
                conn.commit()
                return result
            except sqlite3.Error as e:
                NLog.error(f"execute: {sql}\n{e}")
                conn.rollback()
                return None
            except Exception:
                conn.rollback()
                return None
                # raise e

    @classmethod
    def from_row(cls, row):
        obj = cls()
        for (name, field), value in zip(cls._fields.items(), row):
            setattr(obj, name, value)
        return obj

    @classmethod
    def get_by_id(cls, id):
        if not NHook.isvalid(id):
            return None
        sql = cls.sqlSelect(where="id = ?")

        def callback(cur):
            return cur.fetchone()

        row = cls._execute(sql, (id, ), callback=callback)
        if row:
            return cls.from_row(row)
        return None

    @classmethod
    def get_by_rid(cls, rid):
        if not NHook.isvalid_str(rid):
            return None
        sql = cls.sqlSelect(where="rid = ?")
        row = cls._execute(sql, (rid, ), callback=lambda cur: cur.fetchone())
        if row:
            return cls.from_row(row)
        return None

    @classmethod
    def get(cls, conditions: dict = {}):
        # # 构建条件语句
        # where_conditions = []
        # parameters = []
        # for key, value in conditions.items():
        #     where_conditions.append(f"{key} = ?")
        #     parameters.append(value)

        condition_strings = [f"{key} = ?" for key in conditions.keys()]
        condition_clause = " AND ".join(condition_strings)
        sql = cls.sqlSelect(where=condition_clause)

        params = tuple(conditions.values())
        rows = cls._execute(sql, params, callback=lambda cur: cur.fetchall())
        if rows:
            return [cls.from_row(row) for row in rows]
        return None

    @classmethod
    def query(cls, fields, conditions=None):
        if not NHook.isvalid(fields):
            return None
        field_clause = ', '.join(fields)
        condition_clause = ""
        params = ()
        if conditions:
            condition_strings = [f"{key} = ?" for key in conditions.keys()]
            condition_clause = " WHERE " + " AND ".join(condition_strings)
            params = tuple(conditions.values())

        sql = cls.sqlSelect(field_clause, where=condition_clause)
        rows = cls._execute(sql, params, callback=lambda cur: cur.fetchall())
        if rows:
            return [dict(zip(fields, row)) for row in rows]
        return None

    def save(self):
        if self._exists():
            self._update()
        else:
            self._insert()

    def _insert(self):
        current_time = datetime.now().isoformat(timespec='seconds')
        fields = ', '.join(
            name for name in self._fields.keys()
            if not (self._fields[name].primary_key and self._fields[name].
                    autoincrement) and name not in ['created', 'updated'])
        placeholders = ', '.join(
            '?' for name in self._fields.keys()
            if not (self._fields[name].primary_key and self._fields[name].
                    autoincrement) and name not in ['created', 'updated'])
        values = [
            getattr(self, name) for name in self._fields.keys()
            if not (self._fields[name].primary_key and self._fields[name].
                    autoincrement) and name not in ['created', 'updated']
        ]

        values.append(current_time)  # created
        values.append(current_time)  # updated

        sql = (f"INSERT INTO {self.__class__.__name__.lower()} "
               f"({fields}, created, updated) VALUES ({placeholders}, ?, ?)")
        self.__class__._execute(sql, values)

    def _update(self):
        current_time = datetime.now().isoformat(timespec='seconds')
        fields = ', '.join(f"{name} = ?" for name in self._fields.keys()
                           if name not in ['id', 'created', 'updated'])
        values = [
            getattr(self, name) for name in self._fields.keys()
            if name not in ['id', 'created', 'updated']
        ]
        values.append(current_time)
        values.append(self.id)  # Assuming 'id' is the primary key

        sql = (f"UPDATE {self.__class__.__name__.lower()} SET {fields}, "
               f"updated = ? WHERE id = ?")
        self.__class__._execute(sql, values)

    def _exists(self):
        if not hasattr(self, 'id'):
            return False
        sql = self.__class__.sqlSelect(where="id = ?")
        row = self.__class__._execute(sql, (self.id, ),
                                      callback=lambda cur: cur.fetchone())
        return row is not None



# -*- coding: utf-8 -*-

from ddans.config.time import EXPIRATION_TIME
from ddans.descriptor.singleton import singleton
from ddans.native.time import NTime


@singleton
class SGlobaler(object):

    def __init__(self):
        self._auth_checked = self._auth_check()

    def _auth_check(self):
        expiration_datetime = NTime.time(EXPIRATION_TIME)
        now_datetime = NTime.now()
        return expiration_datetime > now_datetime

    @property
    def auth_checked(self):
        return self._auth_checked



# -*- coding: utf-8 -*-

from typing import Dict, List, Literal, Optional, Union

from ddans.common.type import Element
from ddans.native.html import NHtml


class DHtml:

    def __init__(self):
        self._content = ""

    def list(self,
             list: List[str],
             style: Optional[Dict[str, str | int]] = None,
             el: Literal["ol", "ul"] = "ul",
             type: Literal["1", "i", "a", "A", "i", "I"] = "1"):
        self._content += NHtml.list(list, style, el, type)
        return self

    def element(self,
                content: Union[str, List[str]],
                style: Optional[Dict[str, str | int]] = None,
                attribute: Optional[Dict[str, str]] = None,
                order: Optional[Literal["numeric", "alpha", "chinese"]] = None,
                el: Element = "p"):
        self._content += NHtml.line(content, style, attribute, order, el)
        return self

    def reset(self):
        self._content = ""

    @property
    def content(self):
        return self._content

    @property
    def body(self):
        return NHtml.element(self.content, "body")

    @property
    def html(self):
        return NHtml.element(self.body, "html")



"""Native utilities module"""

from . import brotli
from . import config
from . import ffmpeg
from . import hook
from . import html
from . import log
from . import rsa
from . import string
from . import system
from . import tea
from . import time
from . import url
from . import web

__all__ = [
    "brotli",
    "config",
    "ffmpeg",
    "hook",
    "html",
    "log",
    "rsa",
    "string",
    "system",
    "tea",
    "time",
    "url",
    "web",
]



# -*- coding: utf-8 -*-
from typing import Any, Callable, Literal, Optional, TypeVar, TypedDict

T = TypeVar('T', bound=Optional[Any])

Callback = Callable[..., T]

Element = Literal["p", "ul", "ol", "h1", "h2", "h3", "h4", "h5", "html", 'a',
                  'div', 'span', 'text', 'image', 'video', "body"]

ALL_ROOMS = ["default", "cn", "sgp", "us"]
RoomType = Literal["default", "cn", "sgp", "dc", "us"]

ALL_DEPLOYS = ["test", "prod"]
DeployType = Literal["test", "prod"]


class DataSizeResult(TypedDict):
    total: int
    desc: str
    gb: int
    mb: int
    kb: int
    b: int


class UrlData(TypedDict):
    href: str
    url: str
    path: str
    host: str
    query: str
    scheme: str
    origin: str
    params: dict



# -*- coding: utf-8 -*-

from pathlib import Path
from typing import Any, Dict, Optional

from ddans.native.system import NSystem


class DConfig:

    def __init__(self,
                 path: str = "config.json",
                 default_config: Optional[Dict] = None):
        self.config_path = Path(path)
        self.default_config = default_config or {}
        self._config: Optional[Dict] = None
        self._loaded = False
        self._modified = False

    def load(self, force: bool = False) -> Dict:
        if self._loaded and not force and self._config is not None:
            return self._config.copy()

        self._config = self.default_config.copy()
        if not self.config_path.exists():
            self._loaded = True
            self._modified = True
            self._save_if_needed()
            return self._config.copy()

        cfg = NSystem.read_json(self.config_path, None)
        if cfg:
            self._config = cfg
            self._loaded = True
            self._modified = False
        else:
            self._loaded = True
            self._modified = True
            self._save_if_needed()
        return self._config.copy()

    def _save_if_needed(self) -> None:
        if self._modified and self._config is not None:
            self.save()

    def save(self, force: bool = False) -> bool:
        if self._config is None:
            return False
        if not self._modified and not force:
            return True

        return NSystem.write_json(self._config, self.config_path)

    def get(self, key: str, default: Any = None) -> Any:
        if not self._loaded or self._config is None:
            self.load()

        # 支持多级键
        keys = key.split('.')
        value = self._config

        for k in keys:
            if isinstance(value, dict) and k in value:
                value = value[k]
            else:
                return default

        return value

    def set(self, key: str, value: Any, save: bool = True) -> bool:
        if not self._loaded or self._config is None:
            self.load()

        # 支持多级键
        keys = key.split('.')
        current = self._config

        # 遍历到最后一层
        for k in keys[:-1]:
            if k not in current or not isinstance(current[k], dict):
                current[k] = {}
            current = current[k]

        old_value = current.get(keys[-1])
        if old_value == value:
            return True

        # 设置值
        current[keys[-1]] = value
        self._modified = True

        # 如果需要立即保存
        if save:
            return self.save()

        return True

    def update(self, updates: Dict, save: bool = False) -> bool:
        # 确保配置已加载
        if not self._loaded or self._config is None:
            self.load()

        # 递归更新
        modified = self._update_dict(self._config, updates)

        if modified:
            self._modified = True

        if save and modified:
            return self.save()

        return True

    def _update_dict(self, target: Dict, updates: Dict) -> bool:
        modified = False

        for key, value in updates.items():
            if (key in target and isinstance(target[key], dict)
                    and isinstance(value, dict)):
                # 递归更新嵌套字典
                if self._update_dict(target[key], value):
                    modified = True
            elif key not in target or target[key] != value:
                # 值不同，更新
                target[key] = value
                modified = True

        return modified

    def delete(self, key: str, save: bool = False) -> bool:
        # 确保配置已加载
        if not self._loaded or self._config is None:
            self.load()

        keys = key.split('.')
        current = self._config

        # 找到要删除的项的父级
        for k in keys[:-1]:
            if k in current and isinstance(current[k], dict):
                current = current[k]
            else:
                return False  # 键不存在

        if keys[-1] in current:
            del current[keys[-1]]
            self._modified = True
            if save:
                return self.save()
            return True
        return False

    def reset(self, save: bool = True) -> bool:
        self._config = self.default_config.copy()
        self._loaded = True
        self._modified = True
        if save:
            return self.save()
        return True

    def reload(self) -> Dict:
        return self.load(force=True)

    def get_all(self) -> Dict:
        if not self._loaded or self._config is None:
            self.load()
        return self._config.copy()

    def exists(self, key: str) -> bool:
        if not self._loaded or self._config is None:
            self.load()

        keys = key.split('.')
        value = self._config

        for k in keys:
            if isinstance(value, dict) and k in value:
                value = value[k]
            else:
                return False
        return True

    def ensure_exists(self,
                      key: str,
                      default_value: Any,
                      save: bool = False) -> Any:
        if not self.exists(key):
            self.set(key, default_value, auto_save=save)
            return default_value

        return self.get(key)

    def is_modified(self) -> bool:
        return self._modified

    def auto_save(self) -> bool:
        if self._modified:
            return self.save()
        return True

    def __getitem__(self, key: str) -> Any:
        return self.get(key)

    def __setitem__(self, key: str, value: Any) -> None:
        self.set(key, value, save=False)

    def __contains__(self, key: str) -> bool:
        return self.exists(key)



# -*- coding: utf-8 -*-

import string
from typing import Dict, List, Literal, Optional, Union

from ddans.common.type import Element
from ddans.native.hook import NHook


class NHtml:

    @staticmethod
    def list(list: List[str],
             style: Optional[Dict[str, str | int]] = None,
             el: Literal["ol", "ul"] = "ul",
             type: Literal["1", "i", "a", "A", "i", "I"] = "1"):
        if not NHook.isvalid(list, List) or len(list) <= 0:
            return ""
        content = NHtml.line(list, style, el='li')
        attribute = None if el == 'ul' else {"type": type}
        return NHtml.element(content, el, style, attribute)

    @staticmethod
    def line(content: Union[str, List[str]],
             style: Optional[Dict[str, str | int]] = None,
             attribute: Optional[Dict[str, str]] = None,
             order: Optional[Literal["numeric", "alpha", "chinese"]] = None,
             el: Element = "p"):
        # text-indent: 2em 段落首行缩进两个汉字
        if isinstance(content, str):
            content = [content]

        if not NHook.isvalid(content, List):
            return ""

        result = ""
        for idx, item in enumerate(content):
            prefix = NHtml.order(idx, order)
            result += NHtml.element(f"{prefix}{item}", el, style, attribute)
        return result

    @staticmethod
    def link(href: str, text: str = None):
        url = href or ''
        return NHtml.element(text or url, 'a', attribute={'href': url})

    @staticmethod
    def element(content: str,
                el: Element,
                style: Optional[Dict[str, str | int]] = None,
                attribute: Optional[Dict[str, str]] = None):
        el = el or 'p'
        content = content or ''

        # 创建CSS样式字符串
        style_str = ""
        if style:
            for key, value in style.items():
                style_str += f"{key}: {value};"

        attribute_str = ""
        if attribute:
            for key, value in attribute.items():
                attribute_str += f" {key}='{value}' "

        result = f"<{el} style='{style_str}'{attribute_str}>" if style_str \
            else f"<{el}{attribute_str}>"
        result += f"{content}"
        result += f"</{el}>"
        return result

    @staticmethod
    def order(idx: int,
              order: Literal["numeric", "alpha", "chinese"] = None) -> str:
        if not isinstance(idx, int):
            return ""
        if order == "numeric":
            result = f"{idx + 1}. "
        elif order == "alpha":
            result = f"{string.ascii_lowercase[idx % 26]}. "
        elif order == "chinese":
            result = f"{NHook.number_to_chinese[idx]}、"
        else:
            result = ""
        return result

    @staticmethod
    def markdown_html(content: str, title: str = None):
        return (f"""<!DOCTYPE html>
            <html>
            <head>
                <meta charset="utf-8">
                <meta name="viewport" content="width=device-width, \
                    initial-scale=1">
                <link rel="stylesheet" \
                    href="https://cdnjs.cloudflare.com/ajax/libs/github-markdown-css/5.2.0/github-markdown.min.css">
                <title>{title or ''}</title>
            </head>
            <body>
            <article class="markdown-body">
            {content or ''}
            </article>
            </body>
            </html>
        """)



# -*- coding: utf-8 -*-

from typing import Any
from urllib.parse import parse_qs, urlencode, urljoin, urlparse
from ddans.common.type import UrlData
from ddans.native.hook import NHook


class NUrl:

    @staticmethod
    def join_urls(base_url, *args):
        # 移除每个部分开头和结尾的斜杠，并过滤掉空字符串
        parts = [part.strip('/') for part in args if part]
        # 确保 base_url 以单个斜杠结尾
        base_url = base_url.rstrip('/') + '/'
        full_url = base_url
        for part in parts:
            full_url = urljoin(full_url, part + '/')
        return full_url.rstrip('/')

    @staticmethod
    def url_parse(url: str):
        if not NHook.isvalid_str(url):
            return None
        return NHook.safe_func(urlparse, url, allow_fragments=False)

    @staticmethod
    def url_query(params: Any | None = None, prefix: str = ''):
        try:
            if params is None:
                return ''
            return f'{prefix}{urlencode(params)}'
        except Exception:
            return ''

    @staticmethod
    def query_params(query: str = ""):
        params = parse_qs(query)
        return {key: values[0] for key, values in params.items()}

    @staticmethod
    def parse(url: str):
        href = url
        result: UrlData = {
            'href': href,
            'url': url,
            'path': "",
            'host': '',
            'query': '',
            'scheme': '',
            'origin': '',
            'params': {}
        }

        if not NHook.isvalid_str(href):
            return result
        ret = NHook.safe_func(urlparse, href, allow_fragments=False)
        if ret is None:
            return result
        parsed = dict(ret._asdict())
        result['path'] = parsed.get("path", "")
        result['scheme'] = parsed.get('scheme', "")
        result['host'] = parsed.get('netloc', "")
        result['params'] = parsed.get('params', "")
        query = parsed.get('query', "")
        origin = f"{result['scheme']}://{result['host']}" if NHook.isvalid_str(
            result['scheme']) else result['host']
        result['origin'] = origin
        result['url'] = f"{origin}{result['path']}"
        result['query'] = query
        result['params'] = NUrl.query_params(query)
        return result



# -*- coding: utf-8 -*-

import traceback
import sys
from multiprocessing import freeze_support

from ddans.common.executor import DExecutor
from ddans.data.color import DColor
from ddans.native.log import NLog
from ddans.single.net import SNetter


class Utils():
    net: SNetter = SNetter()
    executor: DExecutor = DExecutor()

    @staticmethod
    def catch(func, *args, **kwargs):
        ret = None
        try:
            ret = func(*args, **kwargs)
            if ret is None:
                ret = True
        except KeyboardInterrupt:
            pass
        except Exception:
            et, ev, tb = sys.exc_info()
            msg = traceback.format_exception(et, ev, tb)
            for m in msg:
                if m.find("catch") < 0:
                    NLog.printf(m, fc=DColor.red)
        finally:
            return ret

    @staticmethod
    def pause():
        return input("Press any key to continue . . .")

    @staticmethod
    def freeze_support():
        freeze_support()



# -*- coding: utf-8 -*-

class StaticProperty:
    def __init__(self, fget=None, fset=None):
        self.fget = fget
        self.fset = fset

    def __get__(self, instance, owner):
        if self.fget is None:
            # raise AttributeError("Unreadable attribute")
            return None
        return self.fget(owner)

    def __set__(self, instance, value):
        if self.fset is None:
            # raise AttributeError("Can't set attribute")
            return
        self.fset(instance.__class__, value)

    def getter(self, fget):
        self.fget = fget
        return self

    def setter(self, fset):
        self.fset = fset
        return self



# -*- coding: utf-8 -*-
from openpyxl import Workbook, load_workbook

from ddans.native.hook import NHook
from ddans.native.system import NSystem
from ddans.native.time import NTime
from ddans.native.log import NLog


class Xlsx():

    @staticmethod
    def write_kv_excel(json: dict | str, excel_file: str):
        try:
            if json is None or excel_file is None or len(excel_file) <= 0:
                return

            if isinstance(json, dict):
                json_data = json
            elif isinstance(json, str) and NSystem.exists(json):
                json_data = NSystem.read_json(json)
            else:
                return
            NSystem.ensure_dir(excel_file)

            wb = Workbook()
            ws = wb.active

            keys = list(json_data.keys())
            for i in range(len(keys)):
                key = keys[i]
                text = json_data[key]
                ws.append([key, text])
            wb.save(excel_file)
        except Exception:
            pass

    @staticmethod
    def write_row_excel(rows: list[list], excel_file: str):
        try:
            if rows is None or excel_file is None or len(excel_file) <= 0:
                return False

            if NHook.is_instance(rows, list[list]):
                rows_data = rows
            else:
                return False
            NSystem.ensure_dir(excel_file)

            wb = Workbook()
            ws = wb.active
            for index in range(len(rows_data)):
                ws.append(rows_data[index])
            wb.save(excel_file)
            return True
        except Exception as e:
            print(f"{e}")
            return False

    @staticmethod
    def write_kv_json(excel: Workbook | str,
                      json_file: str,
                      key_index: int = 0,
                      value_index: int = 1):
        try:
            if excel is None or json_file is None or len(json_file) <= 0:
                return

            if isinstance(excel, Workbook):
                wb = excel
            elif isinstance(excel, str) and NSystem.exists(excel):
                wb = load_workbook(excel)
            else:
                return

            NSystem.ensure_dir(json_file)
            ws = wb.active
            result = {}
            for rows in ws:
                key = rows[key_index].value
                value = rows[value_index].value
                if NHook.isvalid_str(key.strip()) and NHook.isvalid(value):
                    result[key] = value.rstrip('\n')

            jsonData = {
                key: value
                for key, value in result.items() if NHook.isvalid_str(value)
            }
            NSystem.write_json(jsonData, json_file)
        except Exception:
            pass

    @staticmethod
    def write_col_json(excel: Workbook | str, dir: str):
        try:
            if excel is None or dir is None or len(dir) <= 0:
                return

            if isinstance(excel, Workbook):
                wb = excel
            elif isinstance(excel, str) and NSystem.exists(excel):
                wb = load_workbook(excel)
            else:
                return

            ws = wb.active

            max_column = ws.max_column
            names = []
            jsonLists = [{}] * max_column
            for index, row in enumerate(ws):
                key = row[0].value
                if not NHook.isvalid_str(key.strip()):
                    continue
                values = [x.value or '' for x in row]
                if index == 0:
                    names = values
                else:
                    for idx, value in enumerate(values):
                        json = jsonLists[idx] or {}
                        if not NHook.isvalid_str(value):
                            continue
                        json[key] = value.rstrip('\n')
                        jsonLists[idx] = json

            for idx in range(len(names)):
                name = names[idx]
                if not NHook.isvalid_str(name):
                    continue
                if idx >= max_column:
                    break
                json = jsonLists[idx]
                if json is None:
                    continue
                json_file = NSystem.get_full_path(
                    dir, f"{name}_{NTime.format(format='%Y%m%d')}.json")
                NSystem.ensure_dir(json_file)
                jsonData = {
                    key: value
                    for key, value in json.items() if NHook.isvalid_str(value)
                }
                ret = NSystem.write_json(jsonData, json_file)
                if ret:
                    NLog.success(f"{json_file}")
        except Exception:
            pass



# -*- coding: utf-8 -*-
import base64
import hashlib
import json
import re
import types
import uuid

from typing import Any, Dict, List, Type, Union, Tuple, get_args, get_origin
from ddans.common.type import T, Callback, DataSizeResult


class NHook:

    @staticmethod
    def safe_func(func: Callback, *args, **kwargs):
        """_summary_

        Args:
            func (Callable[..., Any]): 方法函数

        Returns:
            Any: 返回值
        """
        result = None
        try:
            result = func(*args, **kwargs)
        except Exception:
            result = None
        finally:
            return result

    @staticmethod
    def buffer_sha512(buffer: types, b64encode=True):
        try:
            if not NHook.isvalid(buffer):
                return None
            sha512 = hashlib.sha512()
            sha512.update(buffer)
            if b64encode:
                digest = sha512.digest()
                return base64.b64encode(digest).decode('utf-8')
            else:
                return sha512.hexdigest()
        except Exception:
            return None

    @staticmethod
    def buffer_md5(buffer: types, b64encode=False):
        try:
            if not NHook.isvalid(buffer):
                return None
            md5 = hashlib.md5()
            md5.update(buffer)
            if b64encode:
                digest = md5.digest()
                return base64.b64encode(digest).decode('utf-8')
            else:
                return md5.hexdigest()
        except Exception:
            return None

    @staticmethod
    def buffer_utf8(buffer: Union[bytes, bytearray, str]):
        try:
            if not NHook.isvalid(buffer, (bytes, bytearray, str), True):
                return None
            if isinstance(buffer, str):
                return buffer
            return buffer.decode('utf-8')
        except Exception:
            return None

    @staticmethod
    def buffer_hex(buffer: Union[bytes, bytearray, str]):
        try:
            if not NHook.isvalid(buffer, (bytes, bytearray, str), True):
                return None
            if isinstance(buffer, str):
                return buffer.encode('utf-8').hex()
            else:
                return buffer.hex()
        except Exception as e:
            print(f'{e}')
            return None

    @staticmethod
    def base64_to_hex(base64_str: str):
        try:
            if not NHook.isvalid(base64_str):
                return None
            buffer = base64.b64decode(base64_str)
            return buffer.hex()
        except Exception:
            return None

    @staticmethod
    def hex_to_base64(hex_str: str) -> str:
        try:
            if not NHook.isvalid(hex_str):
                return None
            # 将十六进制字符串转换为字节
            buffer = bytes.fromhex(hex_str)

            # 将字节数据编码为Base64字符串
            return base64.b64encode(buffer).decode('utf-8')
        except Exception:
            return None

    @staticmethod
    def from_hex(hex_str: str, default=b''):
        try:
            if not NHook.isvalid(hex_str):
                return default
            return bytes.fromhex(hex_str)
        except Exception:
            return default

    @staticmethod
    def from_base64(text: str, default=b''):
        try:
            if not isinstance(text, str):
                return default
            return base64.b64decode(text)
        except Exception:
            return default

    @staticmethod
    def to_base64(buffer: Union[bytes, bytearray, str], default=''):
        try:
            if not NHook.isvalid(buffer, (bytes, bytearray, str), True):
                return default

            if isinstance(buffer, str):
                # 将字符串转换为字节
                buffer = buffer.encode('utf-8')

            # 将字节或字节数组进行Base64编码
            base64_encoded = base64.b64encode(buffer)

            # 将Base64编码的字节转换为字符串
            return base64_encoded.decode('utf-8')
        except Exception:
            return default

    @staticmethod
    def isvalid_base64(base64_str: str) -> bool:
        # 检查字符串是否只包含Base64字符和等号（用于填充）
        base64_pattern = re.compile(r'^[A-Za-z0-9+/]*={0,2}$')

        # 检查长度是否是4的倍数
        if len(base64_str) % 4 != 0:
            return False

        # 检查是否符合Base64字符集
        if not base64_pattern.match(base64_str):
            return False

        try:
            # 尝试解码Base64字符串
            base64.b64decode(base64_str, validate=True)
            return True
        except Exception:
            return False

    @staticmethod
    def convert_value(value, data_type: type = str, default=None):
        try:
            return data_type(value)
        except (ValueError, TypeError):
            # 如果转换失败，则返回 None
            return default

    @staticmethod
    def isvalid_str(value):
        return NHook.isvalid(value, str)

    @staticmethod
    def isvalid(value, value_type: type = None, check_len=False):
        if value is None:
            return False
        if value_type is not None and not isinstance(value, value_type):
            return False
        elif isinstance(value, int):
            return not isinstance(value, bool)
        elif isinstance(value, str):
            return len(value) > 0
        elif isinstance(value, bytes):
            return len(value) > 0

        if check_len:
            return len(value) > 0
        return True

    @staticmethod
    def to_json(text: str | bytes):
        return NHook.safe_func(json.loads, text)

    @staticmethod
    def json_dumps(obj: Any, default=None, indent: int | str = None):
        if obj is None:
            return default
        return json.dumps(obj, ensure_ascii=False,
                          indent=indent).encode('utf-8')

    @staticmethod
    def json_str(obj: Any, default=None, indent: int | str = None):
        if obj is None:
            return default
        return json.dumps(obj, ensure_ascii=False, indent=indent)

    @staticmethod
    def json_format(txt: str = "", indent: int = 2, keep_raw=False):
        try:
            data = json.loads(txt)
            content = json.dumps(data, ensure_ascii=False, indent=indent)
            return content
        except ValueError:
            return txt if keep_raw else ""

    @staticmethod
    def to_list(*args):
        arg_list = []
        for arg in args:
            if isinstance(arg, (list, tuple)):
                arg_list.extend(arg)
            else:
                arg_list.append(arg)
        return arg_list

    @staticmethod
    def to_string(data: dict):
        return NHook.safe_func(str, data)

    @staticmethod
    def to_int(s: str, default: int = 0) -> int:
        try:
            return int(s)
        except ValueError:
            return default

    @staticmethod
    def index(ele: T, lst: List[T] = []) -> int:
        try:
            return lst.index(ele)
        except ValueError:
            return -1

    @staticmethod
    def join_args(*args):
        return " ".join(args)

    @staticmethod
    def to(func: Callback[T], *args, **kwargs) ->\
            Union[Tuple[Exception, None], Tuple[T, None]]:
        try:
            result = func(*args, **kwargs)
            return None, result
        except Exception as e:
            return e, None

    @staticmethod
    def guid() -> str:
        """ 获取guid

        Returns:
            str: guid
        """
        return str(uuid.uuid4())

    @staticmethod
    def uuid32() -> str:
        return NHook.guid().replace("-", "")

    @staticmethod
    def unique(source: list = []):
        """ 列表去重

        Args:
            source (list, optional): 列表. Defaults to [].

        Returns:
            list: 去重后的列表
        """
        return list(dict.fromkeys(source))

    @staticmethod
    def isvalid_version(version):
        """ 版本号是否有效

        Args:
            version (str): 版本号, 如 1.0.0

        Returns:
            bool: 是 | 否
        """
        # 版本号只能包含数字和点，并且点不能在开头或结尾，也不能连续出现
        if not version or version[0] == '.' or version[
                -1] == '.' or '..' in version:
            return False

        parts = version.split('.')

        for part in parts:
            # 检查每个部分是否都是数字
            if not part.isdigit():
                return False

        return True

    @staticmethod
    def compare_versions(version1: str, version2: str):
        """ 版本号对比, 格式如: 1.0.0

        Args:
            version1 (str): 版本号1
            version2 (str): 版本号2

        Returns:
            int: -1 | 0 | 1;
        """
        # 先进行合法性检查
        if not NHook.isvalid_version(version1) or not NHook.isvalid_version(
                version2):
            raise ValueError("Invalid version format")

        # 分割版本号成列表
        v1_parts = version1.split('.')
        v2_parts = version2.split('.')

        # 计算最大长度
        max_length = max(len(v1_parts), len(v2_parts))

        # 补齐版本号部分
        v1_parts += ['0'] * (max_length - len(v1_parts))
        v2_parts += ['0'] * (max_length - len(v2_parts))

        # 逐一比较版本号的部分
        for i in range(max_length):
            v1_part = int(v1_parts[i])
            v2_part = int(v2_parts[i])
            if v1_part > v2_part:
                return 1
            elif v1_part < v2_part:
                return -1

        # 如果所有部分都相同
        return 0

    @staticmethod
    def number_to_chinese(n):

        # 检查输入是否为整数
        if not isinstance(n, int):
            return ''

        # 检查整数是否为非负数
        if n < 0:
            return ''

        # 定义中文数字字符
        num_map = {
            '0': '零',
            '1': '一',
            '2': '二',
            '3': '三',
            '4': '四',
            '5': '五',
            '6': '六',
            '7': '七',
            '8': '八',
            '9': '九'
        }

        # 定义中文单位字符
        unit_map = ['', '十', '百', '千', '万', '十万', '百万', '千万', '亿']

        # 将数字转为字符串并反转以便从低位到高位处理
        num_str = str(n)[::-1]
        chinese_str = ''

        # 遍历数字字符串
        for i in range(len(num_str)):
            digit = num_str[i]
            if digit != '0':
                chinese_str = num_map[digit] + unit_map[i] + chinese_str
            else:
                if i == 0 or num_str[i - 1] != '0':
                    chinese_str = num_map[digit] + chinese_str

        # 去掉末尾的'零'
        if chinese_str.endswith('零'):
            chinese_str = chinese_str[:-1]

        # 处理特殊情况：去掉'一十'的'一'
        if chinese_str.startswith('一十'):
            chinese_str = chinese_str[1:]

        return chinese_str

    @staticmethod
    def is_instance(obj: Any, typ: Type[T]) -> bool:
        origin = get_origin(typ) or typ

        if origin in {dict, Dict}:
            if not isinstance(obj, dict):
                return False

            # Extract key and value types if available, otherwise use Any
            args = get_args(typ)
            key_type, value_type = args if len(args) == 2 else (Any, Any)

            for key, value in obj.items():
                if key_type is not Any and not NHook.is_instance(
                        key, key_type):
                    return False
                if value_type is not Any and not NHook.is_instance(
                        value, value_type):
                    return False
            return True

        # Handle simple built-in types and other generic types
        if isinstance(origin, type):
            if typ == int and isinstance(obj, bool):
                return False
            return isinstance(obj, origin)
        return False

    @staticmethod
    def calc_data_size(data: int) -> DataSizeResult:
        units = [
            {
                'name': 'gb',
                'divisor': 1 << 30
            },  # 1 GB = 2^30 B
            {
                'name': 'mb',
                'divisor': 1 << 20
            },  # 1 MB = 2^20 B
            {
                'name': 'kb',
                'divisor': 1 << 10
            },  # 1 KB = 2^10 B
            {
                'name': 'b',
                'divisor': 1
            },  # 1 B = 1 B
        ]

        result: DataSizeResult = {
            'total': data,
            'desc': '',
            **{
                unit['name']: 0
                for unit in units
            }
        }

        # 计算每个单位的数量
        for unit in units:
            result[unit['name']] = data // unit['divisor']
            data %= unit['divisor']

        # 生成简洁的描述
        if result['gb']:
            result['desc'] = f"{result['gb'] + result['mb'] / 1024:.2f}GB"
        elif result['mb']:
            result['desc'] = f"{result['mb'] + result['kb'] / 1024:.2f}MB"
        elif result['kb']:
            result['desc'] = f"{result['kb'] + result['b'] / 1024:.2f}KB"
        else:
            result['desc'] = f"{result['b']}B"

        return result

    @staticmethod
    def calc_in_format(value, denominator):
        if not denominator:
            return value // 1
        return value // denominator  # 除以1000并向下取整

    @staticmethod
    def get_value_if(condition, true_value, false_value=None):
        # 处理条件（支持函数或静态值）
        cond_result = condition() if callable(condition) else condition
        # 根据条件返回对应的值（支持函数或静态值）
        if cond_result:
            return true_value() if callable(true_value) else true_value
        else:
            return false_value() if callable(false_value) else false_value

    @staticmethod
    def get_value(obj, key: str, default=""):
        try:
            return obj.get(key, default)
        except Exception:
            return default



# -*- coding: utf-8 -*-

from abc import abstractmethod
from ddans.common.type import ALL_DEPLOYS, ALL_ROOMS
from ddans.native.system import ContentType, NSystem
from ddans.native.hook import NHook
from ddans.domain.regex import Regex
from ddans.terminal import Terminal
from ddans.native.log import NLog
from ddans.module.args_parser import ArgsParser


class Cmd(object):

    def __init__(self, cmd='', desc=''):
        self._cmd = cmd
        self._desc = desc
        self._parser = ArgsParser()

    @property
    def cmd(self):
        return self._cmd or ''

    @property
    def desc(self):
        return self._desc or ''

    @property
    def parser(self):
        return self._parser

    def add_argument(self,
                     action: str,
                     short: str = '',
                     type=str,
                     help=None,
                     default=None):
        self.parser.add_argument(action, short, type, help, default)

    def add_help_argument(self):
        self._parser.add_argument("help", "h", type=bool, help="帮助信息")

    def show_usage(self):
        if self.cmd_valid("h", "help"):
            self.parser.usage()
            return True
        return False

    def cmd_value(self, *args, default=None):
        return self.parser.get_value(*args, default=default)

    def cmd_has(self, key):
        return key in self.parser.actions

    def cmd_valid(self, *args):
        return self.parser.get_value(*args, default=False)

    def cmd_export(self, str: ContentType):
        filename = self.parser.get_value(">", default="")
        if not NHook.isvalid_str(filename):
            return
        _, res = NHook.to(NSystem.write_file, str, filename)
        if NHook.isvalid_str(res):
            NLog.success(f">> {res}")

    @abstractmethod
    def exec(self):
        pass

    @staticmethod
    def select_platform():
        return Terminal.select("请选择平台", ["windows", "mac", "linux"])

    @staticmethod
    def select_win_arch():
        return Terminal.select("请选择arch", ["x64", "ia32"])

    @staticmethod
    def select_arch():
        return Terminal.select("请选择arch", ["x64", "arm64"])

    @staticmethod
    def select_deploy():
        return Terminal.select("选择服务器", ALL_DEPLOYS)

    @staticmethod
    def select_room():
        return Terminal.select("请选择机房", ALL_ROOMS)

    @staticmethod
    def select_needupload():
        return Terminal.select_ok("正式上传")

    @staticmethod
    def input_version(title: str = "请输入版本号 (1.0.0): ", default: str = None):
        return Terminal.input_text(title,
                                   pattern=Regex.version,
                                   default=default)

    @staticmethod
    def input_build_version(title: str = "请输入修正号 (0-99): "):
        return Terminal.input_text(title, pattern=Regex.number, default="0")

    @staticmethod
    def input_number(title: str = "请输入数字: ", default="0"):
        return Terminal.input_text(title,
                                   pattern=Regex.number,
                                   default=default)

    @staticmethod
    def select_tempupload():
        return Terminal.select_ok("临时上传")

    @staticmethod
    def select_rebuildresource():
        return Terminal.select_ok("编译资源")

    @staticmethod
    def is_true(cmd):
        if cmd == '1' or cmd is True:
            return True
        else:
            return False



# -*- coding: utf-8 -*-
import datetime
import sqlite3

from ddans.native.system import NSystem
from ddans.native.config import NConfig
from ddans.native.hook import NHook
from ddans.native.log import NLog


class DSqlite(object):

    def __init__(self, name: str):
        if not NHook.isvalid_str(name):
            return

        appData = NConfig.app_data("base")
        dbase = NSystem.join(appData, f"{name}.db")

        NSystem.ensure_dir(dbase)
        self.dbase = dbase

    def create_table(self, sql: str):
        try:

            if not NHook.isvalid_str(sql):
                return False

            # 连接到数据库
            with sqlite3.connect(self.dbase) as conn:
                # 创建一个 cursor 对象
                cur = conn.cursor()

                # # 创建表（如果不存在）
                # cur.execute('''CREATE TABLE IF NOT EXISTS users (
                #     id INTEGER PRIMARY KEY AUTOINCREMENT,
                #     name TEXT NOT NULL,
                #     age INTEGER NOT NULL
                #     email TEXT,
                #     created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                #     updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
                # )''')

                # 创建表
                cur.execute(sql)

                # 检查是否已存在 email 列，如果不存在则添加
                cur.execute("PRAGMA table_info(users)")
                columns = [column[1] for column in cur.fetchall()]
                if 'email' not in columns:
                    cur.execute("ALTER TABLE users ADD COLUMN email TEXT")

            return True
        except sqlite3.Error as e:
            NLog.error(f"create_table: {e}")
            return False
        except Exception:
            return False

    def alter_table(self, name: str, field: str, sql: str = None):
        try:

            if not NHook.isvalid_str(field) or not NHook.isvalid_str(name):
                return

            # 连接到数据库
            with sqlite3.connect(self.dbase) as conn:
                # 创建一个 cursor 对象
                cur = conn.cursor()

                # 检查是否已存在 email 列，如果不存在则添加
                cur.execute(f"PRAGMA table_info({name})")
                columns = [column[1] for column in cur.fetchall()]

                if NHook.isvalid_str(sql):
                    sql = f"ALTER TABLE users ADD COLUMN {field} TEXT"

                if 'field' not in columns:
                    cur.execute(sql)
                    # cur.execute("ALTER TABLE users ADD COLUMN email TEXT")
            return
        except sqlite3.Error as e:
            NLog.error(f"create_table: {e}")
            pass
        except Exception:
            pass

    def execute(self, name: str, sql: str, parameters=()):
        try:

            if not NHook.isvalid_str(sql) or not NHook.isvalid_str(name):
                return

            # 连接到数据库
            with sqlite3.connect(self.dbase) as conn:
                cur = conn.cursor()
                cur.execute(sql, parameters)
                conn.commit()
            return
        except sqlite3.Error as e:
            NLog.error(f"execute: {e}")
            pass
        except Exception:
            pass

    def update(self, name: str, fields: dict, conditions: dict):

        if not NHook.isvalid_str(name) or not NHook.isvalid(
                fields) or not NHook.isvalid(conditions):
            return

        # 构建更新语句
        updates = []
        update_parameters = []
        for key, value in fields.items():
            updates.append(f"{key} = ?")
            update_parameters.append(value)

        # 添加更新时间
        updates.append('updated_at = ?')
        update_parameters.append(datetime.now())

        # 构建条件语句
        where_conditions = []
        condition_parameters = []
        for key, value in conditions.items():
            where_conditions.append(f"{key} = ?")
            condition_parameters.append(value)

        # 构建完整的更新语句
        sql = (f"UPDATE {name} SET {', '.join(updates)} "
               f"WHERE {' AND '.join(where_conditions)}")
        parameters = update_parameters + condition_parameters
        self.execute(name, sql, parameters)



# -*- coding: utf-8 -*-
from typing import Literal


class DBField:

    def __init__(self,
                 field_type: Literal['INTEGER', 'TEXT', 'TIMESTAMP', 'LONG'],
                 primary_key=False,
                 nullable=True,
                 default=None,
                 autoincrement=False):
        self.field_type = field_type
        self.primary_key = primary_key
        self.nullable = nullable
        self.default = default if default is not None else "''" \
            if field_type == 'TEXT' else None
        self.autoincrement = autoincrement



from typing import Any, Dict
from ddans.descriptor.singleton import SingletonMeta


class SManager(metaclass=SingletonMeta):
    def __init__(self) -> None:
        super()
        self._data: Dict[str, Any] = {}

    def set_data(self, key: str, value: Any) -> None:
        self._data[key] = value

    def get_data(self, key: str, default: Any = None) -> Any:
        return self._data.get(key, default)

    def remove_data(self, key: str) -> None:
        if key in self._data:
            del self._data[key]

    def clear_data(self) -> None:
        self._data.clear()



# -*- coding: utf-8 -*-

import asyncio
import inspect
from typing import Tuple, Union
from ddans.common.type import T, Callback


def import_safe(name):
    try:
        # 使用 `importlib` 动态导入模块
        import importlib
        module = importlib.import_module(name)
        return module
    except ImportError:
        return None


def func_safe(func: Callback, *args, **kwargs):
    """_summary_

    Args:
        func (Callable[..., Any]): 方法函数

    Returns:
        Any: 返回值
    """
    result = None
    try:
        result = func(*args, **kwargs)
    except Exception:
        result = None
    finally:
        return result


def func_to(func: Callback[T], *args, **kwargs) ->\
        Union[Tuple[Exception, None], Tuple[T, None]]:
    """_summary_

    Args:
        func (Callable[..., T]): 方法函数

    Returns:
        err, result: 异常，结果
    """
    try:
        result = func(*args, **kwargs)
        return None, result
    except Exception as e:
        return e, None


def func_template(func: Callback[T], args=None, kwargs=None):
    func = func
    args = args if args is not None else []
    kwargs = kwargs if kwargs is not None else {}
    return (func, args, kwargs)


def prints(content: str, chunk_size=1000):
    """分段打印
    Args:
        content (str): 内容
        chunk_size (int, optional): 分段大小. 默认1000.
    """
    length = len(content)
    if length <= chunk_size:
        print(content)
        return

    for i in range(0, length, chunk_size):
        print(content[i:i + chunk_size], end="")
    print()


def run_async(func, *args, **kwargs):
    """安全执行同步或异步函数，支持已有事件循环"""
    if not inspect.iscoroutinefunction(func):
        return func(*args, **kwargs)
    try:
        loop = asyncio.get_running_loop()
    except RuntimeError:
        return asyncio.run(func(*args, **kwargs))
    else:
        return loop.run_until_complete(func(*args, **kwargs))



# -*- coding: utf-8 -*-
import random
import uuid

keyNumber = "0123456789"
keyLower = "abcdefghijklmnopqrstuvwxyz"
keyUpper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
keyChars = keyNumber + keyLower + keyUpper


class NString:

    @staticmethod
    def guid() -> str:
        """ 获取guid

        Returns:
            str: guid
        """
        return str(uuid.uuid4())

    @staticmethod
    def uuid(length: int = 0):
        chars = list(keyChars)
        result = []

        radix = len(chars)
        if length > 0:
            # 生成指定长度的随机ID
            for _ in range(length):
                result.append(chars[random.randint(0, radix - 1)])
        else:
            # 生成标准UUID格式的ID
            result = ['-'] * 36
            result[14] = '4'

            for i in range(36):
                if result[i] == '-':
                    continue
                r = random.randint(0, 15)
                if i == 19:
                    result[i] = chars[(r & 0x3) | 0x8]
                else:
                    result[i] = chars[r]

        return ''.join(result)

    @staticmethod
    def get_hex_string(length=16):
        result = ''.join(random.choice(keyChars) for _ in range(length))
        return result

    @staticmethod
    def to_hex_string(s: str):
        if s is None or len(s) <= 0:
            return ''
        return ''.join(format(ord(c), '02x') for c in s)

    @staticmethod
    def hex_to_string(hex_str: str):
        if hex_str is None or len(hex_str) <= 0:
            return ''
        return bytes.fromhex(hex_str).decode('utf-8')

    @staticmethod
    def to_buffer(txt: str):
        if txt is None or len(txt) <= 0:
            return b''
        return txt.encode("utf-8")

    @staticmethod
    def strip_newlines_from_values(data):
        if isinstance(data, dict):
            return {
                key: NString.strip_newlines_from_values(value)
                for key, value in data.items()
            }
        elif isinstance(data, list):
            return [NString.strip_newlines_from_values(item) for item in data]
        elif isinstance(data, str):
            return data.rstrip('\n')
        else:
            return data



# -*- coding: utf-8 -*-

import gc
import threading


class DGc:
    def __init__(self) -> None:
        self.timer = None
        pass

    def collect(self):
        gc.collect()  # 执行垃圾回收并等待结束
        return self

    def periodic_gc(self, interval):
        self.collect()

        # 使用定时器再次调度下一次垃圾回收
        self.timer = threading.Timer(interval, self.periodic_gc, [interval])
        self.timer.start()
        return self

    def close(self):
        if self.timer and self.timer.is_alive:
            self.timer.cancel()



# -*- coding: utf-8 -*-
from concurrent.futures import FIRST_COMPLETED, ThreadPoolExecutor, wait
from typing import Any, Callable, List

from tqdm import tqdm

from ddans.common.type import T

TQDM_FORMAT = "{l_bar}{bar}| {n_fmt}/{total_fmt}{unit} [{elapsed}]"


class Pbar:

    @staticmethod
    def list(items: List[T],
             func: Callable[[T], Any],
             desc: str = "",
             max=1,
             unit=""):
        futures = []
        with ThreadPoolExecutor(max_workers=max) as executor:
            for item in items:
                futures.append(executor.submit(func, item))
            pbar = tqdm(total=len(futures),
                        desc=desc,
                        disable=False,
                        unit=unit,
                        ncols=80,
                        bar_format=TQDM_FORMAT,
                        colour="cyan")
            while futures:
                done, futures = wait(futures, return_when=FIRST_COMPLETED)
                for future in done:
                    try:
                        future.result()  # 检查异常，但不阻塞
                    except Exception:
                        pass
                    pbar.update(1)
            pbar.close()



# -*- coding: utf-8 -*-
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256
from Crypto.Cipher import PKCS1_OAEP
import base64

from ddans.data.color import DColor
from ddans.native.log import NLog

RSAPublicKey = (
    """MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEApiLWsPmoaiGdr02AmPYS
4/VgxyJNNJSK2VICirXl4P8qAP3b/6GIs7Hwnsmmuj7gledJ0XiZodFjDNgQEZEG
j4K9KX8NixcrJwnTQV5UebiERdj+64ihwqPVWo9dIOj43ZPJ8f4dritJv5tnQaBb
b9zTkU3Ofzk1g8RU00Ith6quOGZcU2IWC4OQ+5YiZvzrCBKEvjFwK2OvGq43tI51
GaZdI9H09jisypqefsMvv6esjEQQOtl+WGlI5fof7KZUt3dppvuDI4CYgeXIoKKL
9T9qDl8F9aNSs3XHx32YyA7L1FraFD6PjqRjueIFp2emsXQcIMeMqqgVyV+4qb+0
uwIDAQAB""")

RSAPrivateKey = (
    """MIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQCmItaw+ahqIZ2v
TYCY9hLj9WDHIk00lIrZUgKKteXg/yoA/dv/oYizsfCeyaa6PuCV50nReJmh0WMM
2BARkQaPgr0pfw2LFysnCdNBXlR5uIRF2P7riKHCo9Vaj10g6Pjdk8nx/h2uK0m/
m2dBoFtv3NORTc5/OTWDxFTTQi2Hqq44ZlxTYhYLg5D7liJm/OsIEoS+MXArY68a
rje0jnUZpl0j0fT2OKzKmp5+wy+/p6yMRBA62X5YaUjl+h/splS3d2mm+4MjgJiB
5cigoov1P2oOXwX1o1KzdcfHfZjIDsvUWtoUPo+OpGO54gWnZ6axdBwgx4yqqBXJ
X7ipv7S7AgMBAAECggEAE1kuWLkSsJ2OoDpt9iJ+a7cJmNd5V1zPWOTwr9fkWFJP
QYNPKlPVOFxKUivY84rcHAkuMyuQ9OQOXZISOQGDWpZW0mzeFmtR1r+Rr2S9gNmI
6huOAL9OW306HUxiA7GuPhv2omy+Zyjac2q1WVLEI2B78Um2+WEfxuNIH7xu0gRR
z/wluevMBQlv6hSeMESP6e9s2kB+FuM4bL7+1sexwS1TudKbHybujaiB8un9E2V3
C9BkE6yf00Rhvii6Q4E4w3PFQfcBEFnjOD0S2Watx8GIbYjlYvtquy8Yj7OtyZB6
w5sxyaYhKVVnkpGAbOgaCj94ABYF47VuwXr45I4PzQKBgQDp6svzD+QKlARjcWgO
rH7UEpFX6m49G6Ib4c63xEg9+GesTgfDxoYxtXDKsC7I8OwMrmffJGbyfjaAvzmg
icM3Y7kF6UPO9+q2mBRsyhoJSNONTeUVyoFSc+QDsVgndlTZIjqC5XLrONGaSPLJ
7jJzqKhp3NQcpO8+xY0d7Efm1QKBgQC10fCNuF1LUTYgsWvvGxTiCCAIBurZmHU+
GMb2Okfoco/P+QNl1jEba91AC7FudW3g/lRn0vTnDajlNoz2/CKunrTKN4/9lEBI
tiwEOvhc5FtapRifSfrrNwFqPM+LijvOkcteWVhAUWCEP1eLmpnXGqfmZxnTaaVe
5DyDgyAVTwKBgQDhEawfOQEANJ9sV2f6BnsZ7quuXcCoR3hZP5G5O8EO735BSesq
sk/YcG4QeCjr6vQU15LWH552YH5srpHyJ9uP5gHW8DTwZZaPDhFJA8S9fjcfVP3B
U9HO1fdknjTl83N8YPTPkZpbPZZfzyTcqrYclwFEqtfkn8YbLdW0Hu6n6QKBgDEv
jvS3uD1ExfDILlurptWSvx+Mx4F/1c+roj8/+u3t9+uaAIpPDE/PuTW3NBOens/r
3WeM0QWkU1lWjYHPKY61jlDaARDnRxCkf3nRfhibCf+Wg/erar6wzagwtTCZf55d
IHfwkYgsu4BrJkK02wMuydbxczIW9U6bTaeCFG7PAoGAN3oHYzTEBwZ7ZKv11QiQ
Ep/HeHBvD8fH/e1uVyBel4Co7NO5pV2s8qjS8+3+jxumKLNXCirjqwHo03YCdzxi
cy4GFDsukHmDQ7FF9VC4QV0fJ0F7jGYE9nNZ+K9qwS5KrihOSBPxj0HptrF6TSOw
BTSwr0byL3TC0m0StkSNKpo=""")


class NRsa:

    @staticmethod
    def rsa_encrypt(source: str, public_pem: str):
        cipher = PKCS1_OAEP.new(key=RSA.import_key(public_pem),
                                hashAlgo=SHA256)
        cipherText = cipher.encrypt(source.encode('utf-8'))
        return base64.b64encode(cipherText).decode('utf-8')

    @staticmethod
    def rsa_decrypt(encrpyted: str, private_pem: str):
        rsaDecrypt = PKCS1_OAEP.new(key=RSA.import_key(private_pem),
                                    hashAlgo=SHA256)
        return rsaDecrypt.decrypt(base64.b64decode(
            encrpyted.encode('utf-8'))).decode('utf-8')

    @staticmethod
    def encode(source: str, publicKey: str = RSAPublicKey):
        _key = NRsa.format_pem_key(publicKey)
        cipher = PKCS1_OAEP.new(key=RSA.import_key(_key), hashAlgo=SHA256)

        dataBytes = source.encode('utf-8')
        encrypted_chunks = []
        chunk_size = 190
        # 按 max_chunk_size 对消息进行分块
        for i in range(0, len(dataBytes), chunk_size):
            chunk = dataBytes[i:i + chunk_size]

            # 加密每一个块
            encrypted_chunk = cipher.encrypt(chunk)

            # 添加加密后的块到列表中
            encrypted_chunks.append(encrypted_chunk)

        # 将所有加密块拼接在一起
        encrypted = b''.join(encrypted_chunks)
        return base64.b64encode(encrypted).decode('utf-8')

    @staticmethod
    def decode(encrpyted: str, privateKey: str = RSAPrivateKey):
        _key = NRsa.format_pem_key(privateKey, "RSA PRIVATE KEY")
        rsaDecrypt = PKCS1_OAEP.new(key=RSA.import_key(_key), hashAlgo=SHA256)

        dataBytes = base64.b64decode(encrpyted.encode('utf-8'))

        chunk_size = 256  # RSA 2048位的块大小
        decrypted_chunks = []

        # 分块处理
        for i in range(0, len(dataBytes), chunk_size):
            chunk = dataBytes[i:i + chunk_size]

            # 解密每一个块
            decrypted_chunk = rsaDecrypt.decrypt(chunk)

            # 将解密后的数据存储
            decrypted_chunks.append(decrypted_chunk)

        # 将解密后的块拼接成完整的字节数组
        decrypted = b''.join(decrypted_chunks)
        return decrypted.decode('utf-8')

    @staticmethod
    def format_pem_key(key_str: str, label: str = 'PUBLIC KEY'):
        # 确保每行64字符
        formatted_key = '\n'.join(
            [key_str[i:i + 64] for i in range(0, len(key_str), 64)])
        # 添加头尾
        pem_key = (f"-----BEGIN {label}-----\n"
                   f"{formatted_key}"
                   f"\n-----END {label}-----")
        return pem_key

    @staticmethod
    def remove_pem(pem: str, label: str = 'PUBLIC KEY'):
        return pem.replace(f"-----BEGIN {label}-----\n",
                           "").replace(f"\n-----END {label}-----",
                                       "").replace("\n", "")

    @staticmethod
    def generate_rsa_key():
        try:
            # Generating private key (RsaKey object) of key length of 1024 bits
            private_key = RSA.generate(2048)
            # Generating the public key (RsaKey object) from the private key
            public_key = private_key.publickey()
            # Converting the RsaKey objects to string
            private_pem = private_key.export_key().decode()
            # -----BEGIN RSA PRIVATE KEY-----
            # xxx
            # -----END RSA PRIVATE KEY-----
            public_pem = public_key.export_key().decode()
            # -----BEGIN PUBLIC KEY-----
            # xxx
            # -----END PUBLIC KEY-----
            private_pem_stripped = NRsa.remove_pem(private_pem,
                                                   "RSA PRIVATE KEY")

            # 去掉公钥的 "BEGIN" 和 "END" 部分
            public_pem_stripped = NRsa.remove_pem(private_pem)
            # 返回包含公钥和私钥的字典对象
            return {
                "private_key": private_pem_stripped,
                "public_key": public_pem_stripped,
                "public_pem": public_pem,
                "private_pem": private_pem,
            }
        except Exception as e:
            NLog.error(f'generate_rsa_key failed! {e}')

    @staticmethod
    def parse_rsa_public_key(private_pem: str):
        try:
            rsa_key = RSA.import_key(private_pem)
            public_key = rsa_key.publickey()
            public_pem = public_key.export_key().decode()
            NLog.printf(f'{public_pem}', fc=DColor.green)
        except Exception as e:
            NLog.error(f'parse_rsa_public_key failed! {e}')



# -*- coding: utf-8 -*-
from typing import List, Union
from pygtrans import Translate, Null
from ddans.native.hook import NHook
from ddans.config.locale import DEFAULT_LANG
from ddans.native.log import NLog
from ddans.native.system import NSystem
from ddans.native.time import NTime


class Gi18n:

    def __init__(self, lang: str = DEFAULT_LANG, source: str = 'auto'):
        self.lang = lang
        self.source = source

    def is_success(self, res):
        if isinstance(res, Null):
            return False
        else:
            return True

    def get_client(self):
        proxy = NSystem.get_proxy()
        if NHook.isvalid_str(proxy):
            return Translate(proxies={'https': f'http://{proxy}'})
        else:
            return Translate()

    def done(self,
             q: Union[str, List[str]],
             lang: str = None,
             source: str = None,
             retries: int = 0):
        try:

            if lang is None:
                lang = self.lang
            if source is None:
                source = self.source

            temps = []
            if isinstance(q, str):
                if NHook.isvalid_str(q):
                    temps = [q]
                else:
                    return None
            elif isinstance(q, list):
                temps = q
            else:
                return None

            lists = [
                item for item in temps if isinstance(item, str)
                and item.strip() != '' and temps.count(item) == 1
            ]

            count = len(lists)
            if count <= 0:
                return None

            map = dict()

            client = self.get_client()
            # 翻译
            results = client.translate(lists, lang, source=source, fmt="text")

            for i in range(min(count, len(results))):
                key = lists[i]
                val = results[i]

                if not self.is_success(val):
                    continue

                txt = val.translatedText
                if NHook.isvalid_str(txt):
                    map[key] = txt
            return map
        except Exception as e:
            if retries > 0:
                NTime.sleep(1)
                return self.done(q, lang, source, retries=retries - 1)
            else:
                NLog.error(f"translate {lang}: {e}")
                return None



# -*- coding: utf-8 -*-

import re


# @dataclass
class Regex:
    version = r'^\d+.\d+.\d+$'
    number = r'^(0|[1-9]\d*)$'
    ip = r'^\d+.\d+.\d+.\d+$'
    # re.findall(r'(\w+)=(\w+)', 'set width=20 and height=10nihao')
    # pattern: str | re.Pattern[str] = None,

    intl = re.compile(r'intl\.t\(\s*([`"\'])([\s\S]*?)\1\s*(?:,|\))',
                      re.DOTALL)

    # 正则表达式模式，用于移除注释
    comment = re.compile(r'//.*?$|/\*[\s\S]*?\*/', re.MULTILINE)

    # 正则表达式模式，用于检查是否包含中文字符
    chinese = re.compile(r'[\u4e00-\u9fff]')

    bracketContent = re.compile(r'(\{.*?\})')

    git_date = re.compile(
        r'^\w{3} \w{3} \d{2} \d{2}:\d{2}:\d{2} \d{4} [+-]\d{4}$')

    gmt_date = re.compile(
        r'^[A-Za-z]{3}, \d{2} [A-Za-z]{3} \d{4} \d{2}:\d{2}:\d{2} GMT$')

    addrport = re.compile(r':\/\/([^:]+):(\d+)')

    id18 = re.compile(r'^\d{18}$')

    re_number = re.compile(number)

    @staticmethod
    def pattern(pattern: str):
        return f"{pattern}"



# -*- coding: utf-8 -*-
import base64
import brotli

from ddans.native.log import NLog


class NBrotli:

    @staticmethod
    def compress(content: str):
        result = ""
        try:
            comporessed = brotli.compress(content.encode('utf-8'))
            result = str(base64.b64encode(comporessed), encoding='utf-8')
        except Exception as e:
            NLog.error(f'brotli compress failed! {e}')
        finally:
            return result

    @staticmethod
    def decompress(comporessed_content: str):
        result = ""
        try:
            comporessed = base64.b64decode(comporessed_content)
            decomporessed = brotli.decompress(comporessed)
            result = decomporessed.decode('utf-8')
        except Exception as e:
            NLog.error(f'brotli compress failed! {e}')
        finally:
            return result



# -*- coding: utf-8 -*-
from Crypto.Cipher import AES
import base64


class Aes:
    def __init__(self):
        self.mode = AES.MODE_CBC

    def pkcs7padding(self, text):
        bs = 16
        length = len(text)
        bytes_length = len(text.encode('utf-8'))
        padding_size = length if (bytes_length == length) else bytes_length
        padding = bs - padding_size % bs
        padding_text = chr(padding) * padding
        self.coding = chr(padding)
        return text + padding_text

    def encode(self, content, key, iv):
        result = ""
        try:
            cipher = AES.new(key.encode("utf-8"),
                             AES.MODE_CBC,
                             iv.encode("utf-8"))
            content_padding = self.pkcs7padding(content)
            encrypt_bytes = cipher.encrypt(content_padding.encode('utf-8'))
            result = str(base64.b64encode(encrypt_bytes), encoding='utf-8')
        except Exception:
            print("加密失败")
        finally:
            return result

    def decode(self, content, key, iv):
        result = ""
        try:
            cipher = AES.new(key.encode("utf-8"),
                             AES.MODE_CBC,
                             iv.encode("utf-8"))
            content = base64.b64decode(content)
            text = cipher.decrypt(content).decode('utf-8')
            result = text.rstrip().rstrip("\x01").\
                rstrip("\x02").rstrip("\x03").rstrip("\x04").rstrip("\x05").\
                rstrip("\x06").rstrip("\x07").rstrip("\x08").rstrip("\x09").\
                rstrip("\x0a").rstrip("\x0b").rstrip("\x0c").rstrip("\x0d").\
                rstrip("\x0e").rstrip("\x0f").rstrip("\x10")
        except Exception:
            print("解密失败")
        finally:
            return result



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta, timezone
import time
from ddans.domain.regex import Regex

# import pytz
# target_timezone = pytz.timezone('Asia/Shanghai')

# from datetime import timezone;

# # 设置为 UTC 时区
# utc_time = utc_time.replace(tzinfo=timezone.utc)

# # 将 UTC 时间转换为东八区时间
# cn_time = utc_time.astimezone(timezone(timedelta(hours=8)))


class NTime():
    MINUTE_S = 60
    HOUR_S = MINUTE_S * 60
    DAY_S = HOUR_S * 24

    # time_str = dt_time.strftime('%Y/%#m/%d')
    # print(time_str)  # 2020/6/17
    @staticmethod
    def format(t: int | float | str | datetime = None,
               format="%Y-%m-%d %H:%M:%S"):

        if isinstance(t, (int | float | str)):
            _dateTime = NTime.time(t)
        elif isinstance(t, datetime):
            _dateTime = t
        else:
            _dateTime = NTime.now()
        if _dateTime is None:
            return ""

        return _dateTime.strftime(format)

    @staticmethod
    def now():
        return datetime.now()

    @staticmethod
    def today():
        return NTime.now().replace(hour=0, minute=0, second=0, microsecond=0)

    @staticmethod
    def timestamp(dtime: datetime = None, delta: timedelta = None):
        # print(time.time())
        if not isinstance(dtime, datetime):
            return int(time.time())
        if isinstance(delta, timedelta):
            dtime += delta
        return int(dtime.timestamp())

    @staticmethod
    def get_time(time_str: str, format: str = '%Y-%m-%d %H:%M:%S'):
        return datetime.strptime(time_str, format)

    @staticmethod
    def time(t: int | float | str):
        """
        时间戳转时间

        参数:
        t: timestamp: 时间戳 | 时间格式 2024-05-17T18:46:37

        返回值:
        datetime: 时间
        """
        try:
            if isinstance(t, str):
                if len(t) > 0:
                    # 判断字符串是否符合模式
                    if Regex.git_date.search(t):
                        return datetime.strptime(t, '%a %b %d %H:%M:%S %Y %z')
                    elif Regex.gmt_date.search(t):
                        # return datetime.strptime(t,
                        #                          '%a, %d %b %Y %H:%M:%S %Z')
                        return NTime.formatGMTime(t)
                    return datetime.fromisoformat(t)
                else:
                    return None

            if not isinstance(t, (int | float)):
                return None

            if t < 0:
                return None
            if t > 1e10:
                t = t / 1000.0
            return datetime.fromtimestamp(t)
        except Exception:
            return None

    @staticmethod
    def formatGMTime(GMTstr: str):
        return datetime.strptime(
            GMTstr, '%a, %d %b %Y %H:%M:%S %Z') + timedelta(hours=8)

    def formatUTCTime(UtcStr: str):
        if not UtcStr.strip():
            return datetime(1970, 1, 1, tzinfo=timezone.utc)
        return datetime.strptime(
            UtcStr, '%Y-%m-%dT%H:%M:%S.%fZ') + timedelta(hours=8)

    # 2026-02-06T06:19:35.000Z

    @staticmethod
    def sleep(seconds: float):
        time.sleep(seconds)



# -*- coding: utf-8 -*-
from ddans.data.color import DColor
from ddans.dfunc import prints
from ddans.domain.format import Format


class NLog():

    @staticmethod
    def log(*args):
        prints(Format.color_string(Format.date_tips(), *args))

    @staticmethod
    def debug(*args):
        prints(Format.color_string(Format.date_tips(), *args))

    @staticmethod
    def info(*args):
        prints(Format.color_string(Format.date_tips(), *args,
                                   fc=DColor.yellow))

    @staticmethod
    def warning(*args):
        prints(Format.color_string(Format.date_tips(), *args,
                                   fc=DColor.yellow))

    @staticmethod
    def error(*args):
        prints(Format.color_string(Format.date_tips(), *args, fc=DColor.red))

    @staticmethod
    def success(*args):
        prints(Format.color_string(Format.date_tips(), *args, fc=DColor.green))

    @staticmethod
    def printf(*args, fc: int = None, date=False):
        if date:
            prints(Format.date_desc(*args, fc=fc))
        else:
            prints(Format.desc(*args, fc=fc))

    @staticmethod
    def tips(*args, bc=DColor.bg_green, date=False):
        prints(Format.tips(*args, bc=bc, date=date))



"""Singleton utilities module"""

from . import emailer
from . import gitter
from . import globaler
from . import logging
from . import manager
from . import net
from . import worker

__all__ = [
    "emailer",
    "gitter",
    "globaler",
    "logging",
    "manager",
    "net",
    "worker",
]



# -*- coding: utf-8 -*-
import urllib.request
import webbrowser
from ddans.native.hook import NHook
from ddans.native.system import NSystem
from ddans.native.url import NUrl


class NWeb:
    @staticmethod
    def get_proxies():
        return urllib.request.getproxies()

    @staticmethod
    def get_proxy(url: str = None):
        try:
            proxies = NWeb.get_proxies()
            if proxies is None:
                proxy = NSystem.get_proxy()
                if NHook.isvalid_str(proxy):
                    return f'http://{proxy}'
                return None
            scheme = "https"
            if NHook.isvalid_str(url):
                info = NUrl.url_parse(url)
                if info is not None:
                    scheme = info.scheme or "https"
            return proxies.get(scheme)
        except Exception:
            return None

    @staticmethod
    def open(url: str = None):
        try:
            if not NHook.isvalid_str(url):
                return False
            return webbrowser.open(url)
        except Exception:
            return False



# -*- coding: utf-8 -*-

import subprocess
from ddans.single.manager import SManager


class SGitter(SManager):

    def get_branch_name(self):
        try:
            # 运行 git 命令获取当前分支名称
            result = subprocess.run(
                ['git', 'rev-parse', '--abbrev-ref', 'HEAD'],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                check=True)
            branch_name = result.stdout.strip()
            return branch_name if branch_name else None
        except subprocess.CalledProcessError:
            # 捕捉到 git 命令错误时返回 None
            return None

    def get_latest_commit_hash(self):
        try:
            # 最近提交的 hash 值
            result = subprocess.run(['git', 'rev-parse', 'HEAD'],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    text=True,
                                    check=True)
            return result.stdout.decode('utf-8').strip()
        except subprocess.CalledProcessError:
            # 捕捉到 git 命令错误时返回 None
            return ""

    def get_commit_details(commit_hash):
        try:
            # 最近提交的 hash 值
            result = subprocess.run(
                ['git', 'show', '-s', '--format=%cn,%ce,%s', commit_hash],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                check=True)
            details = result.stdout.decode('utf-8').strip().split(',')
            return {
                'name': details[0],
                'email': details[1],
                'message': ','.join(details[2:])
            }
        except subprocess.CalledProcessError:
            # 捕捉到 git 命令错误时返回 None
            return None

    @property
    def branch_name(self):
        return self.get_data("branch_name")

    def update_branch(self):
        name = self.get_branch_name()
        self.set_data("branch_name", name)



# -*- coding: utf-8 -*-
from collections import deque


class Queue():

    def __init__(self):
        super()
        self.queue = deque()

    def enqueue(self, item):
        self.queue.append(item)

    def dequeue(self):
        if not self.is_empty():
            return self.queue.popleft()
        else:
            raise IndexError("dequeue from empty queue")

    def peek(self):
        if not self.is_empty():
            return self.queue[0]
        else:
            raise IndexError("peek from empty queue")

    def is_empty(self):
        return len(self.queue) == 0

    def size(self):
        return len(self.queue)



# -*- coding: utf-8 -*-
import uuid
from ddans.dbase.field import DBField
from ddans.dbase.model import DBModel


class DBase(DBModel):

    _fields = {
        'id':
        DBField('INTEGER',
                primary_key=True,
                autoincrement=True,
                nullable=False),
        'rid':
        DBField('TEXT', nullable=False),
        'tag':
        DBField('TEXT', default="''"),
        'desc':
        DBField('TEXT', default="''"),
        'memo':
        DBField('TEXT', default="''"),
        'created':
        DBField('TIMESTAMP'),
        'updated':
        DBField('TIMESTAMP')
    }

    def __init__(self, **kwargs):
        self.rid = str(uuid.uuid4())
        for key, value in kwargs.items():
            if key in self._fields:
                setattr(self, key, value)
        for field_name, field in self._fields.items():
            if not hasattr(self, field_name):
                setattr(self, field_name, field.default)



# -*- coding: utf-8 -*-
import threading


class DEvent:
    def __init__(self):
        self.listeners = {}
        self.lock = threading.Lock()

    def on(self, eventName, callback):
        with self.lock:
            if callback is None or not callable(callback):
                return
            if eventName not in self.listeners:
                self.listeners[eventName] = [callback]
            elif callback not in self.listeners[eventName]:
                self.listeners[eventName].append(callback)
            else:
                print(f"Callback '{callback}' is already subscribed to event \
'{eventName}'")

    def off(self, eventName, callback):
        with self.lock:
            if eventName not in self.listeners:
                print(f"No subscribers found for event '{eventName}'")
                return
            if callback in self.listeners[eventName]:
                self.listeners[eventName].remove(callback)
            else:
                print(f"Callback '{callback}' is not subscribed to event \
'{eventName}'")

    def emit(self, eventName, *args, **kwargs):
        with self.lock:
            if eventName in self.listeners:
                for callback in self.listeners[eventName]:
                    try:
                        if callback is None or not callable(callback):
                            continue
                        callback(*args, **kwargs)
                    except Exception as e:
                        print(f"An error occurred while executing callback \
for event '{eventName}': {e}")
            else:
                print(f"No subscribers found for event '{eventName}'")



# -*- coding: utf-8 -*-
import base64
import struct

delta = 0x9e3779b9

TEAHexKey = "33576f37387276645054657a6831686e454664424e5652456f47794835795942"

TEAKey = bytes.fromhex(TEAHexKey).decode('utf-8')


class NTea:

    @staticmethod
    def to_tea_key(key_str: str):
        key_bytes = key_str.encode('utf-8')
        key = [0] * 4
        for i in range(4):
            key[i] = struct.unpack('>I', key_bytes[i * 4:(i + 1) * 4])[0]
        return key

    @staticmethod
    def encrypt(v: list[int], tea_key: list[int]):
        """TEA 加密"""
        sum = 0
        n = 32  # 32轮加密
        for _ in range(n):
            sum = (sum + delta) & 0xffffffff
            v[0] = (v[0] + (((v[1] << 4) + tea_key[0]) ^ (v[1] + sum) ^
                            ((v[1] >> 5) + tea_key[1]))) & 0xffffffff
            v[1] = (v[1] + (((v[0] << 4) + tea_key[2]) ^ (v[0] + sum) ^
                            ((v[0] >> 5) + tea_key[3]))) & 0xffffffff
        return v

    @staticmethod
    def decrypt(v: list[int], tea_key: list[int]):
        """TEA 解密"""
        n = 32  # 32轮解密
        sum = (delta * n) & 0xffffffff
        for _ in range(n):
            v[1] = (v[1] - (((v[0] << 4) + tea_key[2]) ^ (v[0] + sum) ^
                            ((v[0] >> 5) + tea_key[3]))) & 0xffffffff
            v[0] = (v[0] - (((v[1] << 4) + tea_key[0]) ^ (v[1] + sum) ^
                            ((v[1] >> 5) + tea_key[1]))) & 0xffffffff
            sum = (sum - delta) & 0xffffffff
        return v

    @staticmethod
    def pkcs7_padding(plaintext: bytes, block_size=8):
        """PKCS#7 填充"""
        padding = block_size - (len(plaintext) % block_size)
        return plaintext + bytes([padding] * padding)

    @staticmethod
    def pkcs7_unpadding(ciphertext: bytes):
        """PKCS#7 去除填充"""
        padding = ciphertext[-1]
        return ciphertext[:-padding]

    @staticmethod
    def bytes_to_uint32(bytes_seq: bytes) -> list[int]:
        return list(struct.unpack('>II', bytes_seq))

    @staticmethod
    def uint32_to_bytes(uint32_seq: list[int]):
        return struct.pack('>II', *uint32_seq)

    @staticmethod
    def encode(plaintext: str, key: str = TEAKey):
        padded = NTea.pkcs7_padding(plaintext.encode("utf-8"))
        tea_key = NTea.to_tea_key(key)

        encrypted_blocks = []

        for i in range(0, len(padded), 8):
            block = NTea.bytes_to_uint32(padded[i:i + 8])
            encrypted_block = NTea.encrypt(block, tea_key)
            encrypted_blocks.append(NTea.uint32_to_bytes(encrypted_block))

        encrypted = b''.join(encrypted_blocks)
        return base64.b64encode(encrypted).decode('utf-8')

    @staticmethod
    def decode(ciphertext: str, key: str = TEAKey):
        cipherbytes = base64.b64decode(ciphertext)
        tea_key = NTea.to_tea_key(key)
        decryptedBlocks = []

        for i in range(0, len(cipherbytes), 8):
            block = NTea.bytes_to_uint32(cipherbytes[i:i + 8])
            decryptedBlock = NTea.decrypt(block, tea_key)
            decryptedBlocks.append(NTea.uint32_to_bytes(decryptedBlock))

        decrypted = b''.join(decryptedBlocks)
        return NTea.pkcs7_unpadding(decrypted).decode("utf-8")



# -*- coding: utf-8 -*-

import os
import platform
import re
import sys
from typing import List
from ddans.data.color import DColor
from ddans.domain.format import Format
from ddans.native.hook import NHook
from ddans.native.log import NLog
from ddans.native.system import NSystem

if NSystem.is_darwin:
    import readline  # noqa: F401,E111
else:
    import ctypes  # noqa: F401,E111

CmdExit = "exit"
CmdReset = "reset"


class Terminal(object):

    @staticmethod
    def input(desc="", fc: int = None, date=True):
        try:
            text = Format.desc(desc, date=False, fc=fc)
            if date:
                return input(Format.date_tips(text)).strip()
            else:
                return input(text).strip()
        except KeyboardInterrupt:
            print("\n")
            return ""
        finally:
            pass

    @staticmethod
    def print(desc="", fc: int = None, date=False):
        text = Format.desc(desc, date=False, fc=fc)
        if date:
            return print(Format.date_tips(text))
        else:
            return print(text)

    @staticmethod
    def pause():
        return Terminal.input("Press any key to continue . . .")

    @staticmethod
    def is_exit(desc=""):
        return desc.lower() == "quit" or desc.lower() == "exit"

    @staticmethod
    def is_reset(desc=""):
        return desc.lower() == "reset"

    @staticmethod
    def is_cmd(desc=""):
        if Terminal.is_exit(desc):
            return True
        elif Terminal.is_reset(desc):
            return True
        else:
            return False

    @staticmethod
    def get_cmd(desc=""):
        if Terminal.is_exit(desc):
            return CmdExit
        elif Terminal.is_reset(desc):
            return CmdReset
        else:
            return ""

    @staticmethod
    def input_cmd(title: str,
                  map: dict = dict(),
                  tips: str = "选择有误, 请重新选择",
                  fc: int = DColor.yellow,
                  only_key=False):
        only_input = len(map) <= 0
        while True:
            _input = Terminal.input(title, fc)
            if Terminal.is_cmd(_input):
                return _input
            elif len(_input) <= 0:
                NLog.error(tips)
            elif map.get(_input) is not None:
                return map.get(_input)
            elif only_input and len(_input) > 0:
                return _input
            elif not only_key and _input in map.values():
                return _input
            else:
                NLog.error(tips)

    @staticmethod
    def select(title: str,
               options: List[str] = [],
               tips: str = "选择有误, 请重新选择",
               fc: int = DColor.yellow):
        _len = len(options)
        _title = f"{title}: "
        if _len > 0:
            lists = [f"{i+1}: {options[i]}" for i in range(_len)]
            _desc = Format.desc(f"({', '.join(lists)})", DColor.red)
            _title = f"{title} {_desc}: "
        while True:
            _input = Terminal.input(_title, fc)
            _num = int(_input) if _input.isdigit() else 0
            if Terminal.is_cmd(_input):
                return _input
            elif len(_input) <= 0:
                NLog.error(tips)
            elif _num > 0 and _num <= _len:
                return options[_num - 1]
            elif _input in options:
                return _input
            elif _len <= 0 and len(_input) > 0:
                return _input
            else:
                NLog.error(tips)

    @staticmethod
    def select_num(title: str,
                   options: List[str] = [],
                   tips: str = "选择有误, 请重新选择",
                   fc: int = DColor.yellow):
        cmd = Terminal.select(title, options, tips, fc)
        if cmd in options:
            return options.index(cmd) + 1
        else:
            if len(Terminal.get_cmd(cmd)) > 0:
                return 0
            else:
                return -1

    @staticmethod
    def select_ok(title: str,
                  tips: str = "选择有误, 请重新选择",
                  fc: int = DColor.yellow):
        options = ["否", "是"]
        cmd = Terminal.select(title, ["否", "是"], tips, fc)
        if cmd in options:
            return str(options.index(cmd))
        else:
            return cmd

    @staticmethod
    def input_text(title: str,
                   tips: str = "输入有误, 请重新输入",
                   pattern: str | re.Pattern[str] = None,
                   fc: int = DColor.yellow,
                   default: str = None):
        while True:
            _input = Terminal.input(title, fc)
            if Terminal.is_cmd(_input):
                return _input
            elif pattern is None:
                return _input
            elif len(_input) <= 0 and NHook.isvalid_str(default):
                return default
            elif (pattern is not None and len(pattern) > 0
                  and re.match(pattern, _input)):
                return _input
            else:
                NLog.error(tips)

    @staticmethod
    def system_cmd(cmd: str):
        if len(cmd) <= 0:
            return

        if NSystem.is_windows and cmd == 'clear':
            cmd = 'cls'
        os.system(cmd)

    @staticmethod
    def ensure_cmd(title: str = "确定执行该操作吗？"):
        _desc = Format.desc("(yes|no)", DColor.red)
        _title = f"{title} {_desc}: "
        return Terminal.input_cmd(
            _title, {
                "0": "exit",
                "1": "yes",
                "no": "exit",
                "yes": "yes",
                "n": "exit",
                "y": "yes"
            })

    @staticmethod
    def set_title(title: str):
        try:
            if not NHook.isvalid_str(title):
                return

            system = platform.system()
            if system == "Windows":
                ctypes.windll.kernel32.SetConsoleTitleW(f"{title}")
            elif system == "Darwin" or system == "Linux":
                sys.stdout.write(f"\033]0;{title}\007")
                # 修改 iTerm2 标签名称
                sys.stdout.write(f"\033]1;{title}\007")
                sys.stdout.flush()
            else:
                pass
        except Exception:
            pass



# -*- coding: utf-8 -*-
import logging
import sys
from time import strftime

from ddans.native.system import NSystem
from ddans.descriptor.singleton import singleton

# 设置日志格式#和时间格式
FMT = '[%(asctime)s] [%(levelname)s] [%(filename)s] [%(funcName)s]: \
%(message)s'

DATEFMT = '%Y-%m-%d %H:%M:%S'


@singleton
class SLogging(object):

    def __init__(self):
        self.logger = logging.getLogger()
        self.formatter = logging.Formatter(fmt=FMT, datefmt=DATEFMT)
        self.log_filename = NSystem.get_full_path(
            "logs", '%s.log' % strftime("%Y-%m-%d"))
        NSystem.ensure_dir(self.log_filename)
        self.logger.addHandler(self.get_file_handler(self.log_filename))
        # self.logger.addHandler(self.get_console_handler())
        # 设置日志的默认级别
        self.logger.setLevel(logging.DEBUG)

    # 输出到文件handler的函数定义
    def get_file_handler(self, filename):
        filehandler = logging.FileHandler(filename, encoding="utf-8")
        filehandler.setFormatter(self.formatter)
        return filehandler

    # 输出到控制台handler的函数定义
    def get_console_handler(self):
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(self.formatter)
        return console_handler


SLogger: logging.RootLogger = SLogging().logger



"""Common utilities module"""

from . import event
from . import executor
from . import gc
from . import html
from . import type

__all__ = [
    "event",
    "executor",
    "gc",
    "html",
    "type",
]



# -*- coding: utf-8 -*-
from ddans.data.color import DColor
from ddans.native.time import NTime


class Format():
    DATE_TIME = '%Y%m%d%H%M%S'

    @staticmethod
    def color_string(*args, fc: int = None):
        return DColor.text(*args, fc=fc)

    @staticmethod
    def date_string(text="", show=True):
        if show and len(text) > 0:
            return "[%s] %s" % (NTime.format(), text)
        elif show:
            return "[%s]" % NTime.format()
        else:
            return text

    @staticmethod
    def date_tips(text=""):
        tips = DColor.text(f"[{NTime.format()}]", fc=DColor.white, bc=58, sc=1)
        if len(text) > 0:
            return "%s %s" % (tips, text)
        else:
            return "%s" % tips

    @staticmethod
    def time_tips(text=""):
        tips = DColor.text(f"[{NTime.format(format='%H:%M:%S')}]",
                           fc=DColor.white,
                           bc=58,
                           sc=1)
        if len(text) > 0:
            return "%s %s" % (tips, text)
        else:
            return "%s" % tips

    # @staticmethod
    # def test():
    #     for i in range(0, 256):
    #         tips = DColor.text(f"[{NTime.format()}]",
    #                            fc=DColor.white,
    #                            bc=i,
    #                            sc=1)
    #         print(f"{tips} {i}\n")

    @staticmethod
    def desc(context, fc: int = None, date=False, sc: int = None):
        return DColor.text(Format.date_string(context, show=date),
                           fc=fc,
                           sc=sc)

    @staticmethod
    def tips(context, bc: int = DColor.bg_green, date=False, sc: int = None):
        return DColor.text(Format.date_string(context, show=date),
                           bc=bc,
                           fc=DColor.white,
                           sc=sc)

    @staticmethod
    def date_desc(context, fc: int = None):
        return Format.desc(context, fc, date=True)

    @staticmethod
    def upper_first(txt: str):
        if not isinstance(txt, str):
            return ""
        return '{}{}'.format(txt[:1].upper(), txt[1:])



# -*- coding: utf-8 -*-
from functools import wraps
import threading
from typing import Any, Dict, Type, TypeVar

T = TypeVar('T')


class SingletonMeta(type):
    _instances = {}
    _lock = threading.Lock()

    def __call__(cls, *args, **kwargs):
        with cls._lock:
            if cls not in cls._instances:
                cls._instances[cls] = super().__call__(*args, **kwargs)
            return cls._instances[cls]


def singleton(cls):
    instances: Dict[Type[T], T] = {}
    lock = threading.Lock()

    @wraps(cls)
    def get_instance(*args: Any, **kwargs: Any) -> T:
        nonlocal instances
        nonlocal lock
        with lock:
            if cls not in instances:
                instances[cls] = cls(*args, **kwargs)
            return instances[cls]

    return get_instance



# -*- coding: utf-8 -*-
import io
from PIL import Image
import icnsutil
import os

from ddans.native.system import NSystem
from ddans.native.hook import NHook
from ddans.utils import Utils

SUPPORT_TYPE = ["png", "jpg"]
ICON_SIZES = [32, 64, 128, 256, 512, 1024]
IMAGE_SIZES = [16, 24, 32, 48, 64, 128, 256, 512, 1024]
HEADER_SIZE = 8

ICNS_SIZES = [128, 256, 512, 32, 64, 256, 512, 1024]
ICNS_SIZE_TYPES = ['ic07', 'ic08', 'ic09',
                   'ic11', 'ic12', 'ic13', 'ic14', 'ic10']


def toInt(s):
    b = s.encode('ascii')
    return (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3]


class Icon:
    name = ""
    ext = ""
    file = ""
    dirname = ""
    is_valid = False
    image = None

    def __init__(self, file):
        self.file = file
        self.dirname = os.path.dirname(file)
        name, ext = NSystem.split_name_ext(file)
        self.name = name
        self.ext = ext
        if self.ext in SUPPORT_TYPE and os.path.exists(file):
            self.image = Image.open(self.file)
            self.is_valid = True

    def convert(self,
                format: str = None,
                width: int = 0,
                height: int = 0,
                name: str = "",
                dest: str = ""):
        if not self.is_valid or self.image is None:
            return None
        if format is None or len(format) <= 0:
            return None

        _format = format.lower()
        _name = name if len(name) > 0 else self.name
        if _format == self.ext and _name == self.name:
            return None

        _io = self.to_bytes(format=format, width=width, height=height)
        if _io is None:
            return None

        dirname = dest if NHook.isvalid_str(dest) else self.dirname
        outName = NSystem.get_full_path(dirname, f"{_name}.{_format}")
        return Utils.catch(
            lambda: NSystem.write_file(_io.getvalue(), outName))

    def to_bytes(self, format: str = None, width: int = 0, height: int = 0):
        if not self.is_valid or self.image is None:
            return None
        if format is None or len(format) <= 0:
            return None

        _format = format.lower()
        _io = io.BytesIO()

        w = width if width > 0 else self.image.width
        h = height if height > 0 else self.image.height
        im = self.image.resize(size=(w, h))

        if _format == "icns":
            ret = Utils.catch(
                lambda: im.save(_io, format=self.fix_format(_format)))
        elif _format == "ico" and (w >= 512 or h >= 512):
            return None
        else:
            ret = Utils.catch(
                lambda: im.save(_io, format=self.fix_format(_format)))
        if ret is not None:
            return _io
        else:
            return None

    def fix_format(self, fmt: str):
        format = fmt
        if fmt in ["jp2", "jpx", "j2k"]:
            format = "jpeg2000"
        return format

    def to_ico(self, width=0, height=0, name="", filename=""):
        if not self.is_valid:
            return None
        dest = ""
        if NHook.isvalid_str(filename):
            dest = NSystem.get_dir(filename)
            name, _ = NSystem.split_name_ext(filename)
        return self.convert("ico", width, height, name, dest)

    def to_icns(self, name="", filename=""):
        if not self.is_valid:
            return None
        dest = ""
        if NHook.isvalid_str(filename):
            dest = NSystem.get_dir(filename)
            name = NSystem.split_name_ext(filename)
        result = self.convert("icns", name=name, dest=dest)
        self.to_jp2_icns(name, suffix="512")
        return result

    def to_jp2_icns(self, name: str = "", size=512, suffix="", dest=""):
        if not self.is_valid:
            return

        _size = size if size > 0 else 512
        _idx = ICNS_SIZES.index(_size)
        _idx = _idx if _idx >= 0 else ICON_SIZES.index(512)
        if _idx < 0:
            _size = 512
            _idx = ICON_SIZES.index(_size)

        _tytes_io = self.to_bytes("jp2", width=_size, height=_size)
        if _tytes_io is None:
            return
        img = icnsutil.IcnsFile()
        img.add_media(ICNS_SIZE_TYPES[_idx], data=_tytes_io.getvalue())

        _name = name if len(name) > 0 else self.name
        _name_ext = "%s%s%s.icns" % (
            _name,
            "_" if len(suffix) > 0 else "",
            suffix)

        dirname = dest if NHook.isvalid_str(dest) else self.dirname
        outName = NSystem.get_full_path(dirname, _name_ext)
        NSystem.ensure_dir(outName)
        Utils.catch(lambda: img.write(outName))

    def to_icons(self, dest=""):
        if not self.is_valid:
            return

        dirname = dest if NHook.isvalid_str(dest) else self.dirname
        dir = os.path.join(dirname, "%s_icons" % self.name)

        # png
        for size in IMAGE_SIZES:
            _name = "%s_%sx%s" % (self.name, size, size)
            self.convert("png",
                         width=size, height=size, name=_name, dest=dir)

        # ico
        for size in IMAGE_SIZES:
            _name = os.path.join(
                "ico",
                "%s_%sx%s.ico" % (self.name, size, size))
            filename = NSystem.get_full_path(dir, _name)
            self.to_ico(width=size, height=size, filename=filename)
        # icns
        self.convert("icns", dest=dir)
        Utils.catch(
            lambda: self.to_jp2_icns(suffix="512", dest=dir))

    def icns_extract(self, dest=""):
        if not os.path.exists(self.file):
            return None
        format = self.ext.lower()
        if format != "icns":
            return None
        icns = icnsutil.IcnsFile(self.file)
        if icns is None:
            return None

        dirname = dest if NHook.isvalid_str(dest) else self.dirname
        outDir = NSystem.get_full_path(dirname, f"{self.name}_icns")
        NSystem.ensure_dir(outDir)
        icns.export(outDir, allowed_ext='jp2', recursive=True)
        icns.export(outDir, allowed_ext='png', recursive=True,
                    convert_png=True)
        return outDir



# -*- coding: utf-8 -*-
import queue
import threading
import contextlib

StopEvent = object()


class DExecutor(object):

    def __init__(self, maxsize=1):
        self.q = queue.Queue()
        self.maxsize = maxsize
        self.terminal = False
        self.generate_list = []
        self.free_list = []
        self.lock = threading.Lock()

    def generate_thread(self):
        t = threading.Thread(target=self.call)
        t.start()

    @contextlib.contextmanager
    def worker_state(self, lst, t):
        lst.append(t)
        try:
            yield
        finally:
            lst.remove(t)

    def close(self):
        with self.lock:
            num = len(self.generate_list)
            while num:
                self.q.put(StopEvent)
                num -= 1

    def terminate(self):
        with self.lock:
            self.terminal = True
            while self.generate_list:
                self.q.put(StopEvent)

    def submit(self, func, *args, callback=None):
        """
        提交一个任务

        func: 任务函数。

        args: 任务函数所需的参数。

        callback: 任务执行完毕后调用的回调函数，回调函数有两个参数:
                         1、任务函数执行状态（True表示成功，False表示失败）；
                         2、任务函数返回值或异常。

        return: 如果线程池正在终止，则返回True，否则返回None。
        """
        if self.terminal:
            return True

        if len(self.free_list) == 0 and len(self.generate_list) < self.maxsize:
            self.generate_thread()

        task = (func, args, callback)
        self.q.put(task)

    def run(self, func, *args):
        """
        执行一个任务，并等待结果
        func: 任务函数。

        args: 任务函数所需的参数。

        return: 状态, 结果值
        """
        if self.terminal:
            return False, None

        result_queue = queue.Queue()

        def wrapper(status, result):
            result_queue.put((status, result))

        self.run(func, *args, callback=wrapper)

        status, result = result_queue.get()
        return status, result

    def call(self):
        current_thread = threading.currentThread()
        with self.lock:
            self.generate_list.append(current_thread)

        while True:
            event = self.q.get()
            if event == StopEvent:
                break

            func, arguments, callback = event
            try:
                result = func(*arguments)
                status = True
            except Exception as e:
                status = False
                result = e

            if callback:
                try:
                    callback(status, result)
                except Exception:
                    pass

            if self.terminal:
                break

            with self.worker_state(self.free_list, current_thread):
                if self.terminal:
                    break

        with self.lock:
            self.generate_list.remove(current_thread)



# -*- coding: utf-8 -*-

from ddans.native.hook import NHook
from ddans.terminal import Terminal

DEF_ARGS = ['>']


class ArgsParser:

    def __init__(self):
        self.args = {}
        self.actions = {}
        self.types = {}
        for arg in DEF_ARGS:
            self.add_def(arg)

    def __getitem__(self, key):
        return self.actions.get(key)

    def __getattr__(self, attr):
        return self.get_val(attr)

    def get_value(self, *args, default=None):
        if (len(args) <= 0):
            return default

        key = args[0]
        count = len(args[1:])
        _val = self.get_val(key) if NHook.isvalid_str(key) else None
        if _val is None:
            return default if count <= 0 else self.get_value(*args[1:],
                                                             default=default)
        elif key in self.args and NHook.is_instance(
                _val, self.args[key].get('type')):
            return _val
        else:
            return default if count <= 0 else self.get_value(*args[1:],
                                                             default=default)

    def get_bool(self, *args):
        return self.get_value(*args, default=False)

    def get_val(self, key):
        return self.actions.get(key) or self.get_default(key)

    def get_default(self, key):
        if key in self.args:
            return self.args.get(key).get('default')
        else:
            return None

    def add_argument(self,
                     action: str,
                     short: str = '',
                     type=str,
                     help=None,
                     default=None):
        if not NHook.isvalid_str(action):
            return

        _short = short if NHook.isvalid_str(short) else None
        self.args[action] = {
            'type': type,
            'help': help,
            'action': action,
            'short': _short,
            'is_short': False,
            'is_def': False,
            'default': default
        }
        self.types[action] = type

        if _short is None:
            return

        self.args[_short] = {
            'type': type,
            'help': help,
            'action': _short,
            'short': None,
            'is_short': True,
            'is_def': False,
            'default': default
        }
        self.types[_short] = type

    def add_def(self, action: str, type=str, default=''):
        self.args[action] = {
            'type': type,
            'help': '',
            'action': action,
            'short': None,
            'is_short': False,
            'is_def': True,
            'default': default,
        }
        self.types[action] = type

    def reset(self):
        self.actions = {}

    def parse(self, *args):
        self.reset()
        argv = NHook.to_list(args)
        current_arg = None

        # 遍历命令行参数列表
        for i, arg in enumerate(argv, start=0):
            # 如果参数以 - 或 -- 开头
            if arg.startswith('-'):
                # 如果参数以 -- 开头
                if arg.startswith('--'):
                    # 去掉开头的 --，得到参数名
                    current_arg = arg[2:]
                    # 如果参数包含等号，则提取参数值
                    if '=' in current_arg:
                        name, value = current_arg.split('=', 1)
                        self.actions[name] = self._convert_value(name, value)
                    else:
                        self.actions[current_arg] = True  # 没有等号，则视为布尔标志参数
                        current_arg = None
                # 如果参数以 - 开头
                else:
                    # 去掉开头的 -，得到参数名
                    current_arg = arg[1:]
                    # 如果下一个参数不以 - 开头且不是最后一个参数，则作为参数值
                    if i < len(argv) - 1 and not argv[i + 1].startswith('-'):
                        self.actions[current_arg] = self._convert_value(
                            current_arg, argv[i + 1])
                        current_arg = None
                    else:
                        self.actions[current_arg] = True
            elif DEF_ARGS.count(arg) > 0:
                current_arg = arg
                # 如果下一个参数不以 - 开头且不是最后一个参数，则作为参数值
                if i < len(argv) - 1 and not argv[i + 1].startswith('-'):
                    self.actions[current_arg] = self._convert_value(
                        current_arg, argv[i + 1])
                    current_arg = None
                else:
                    self.actions[current_arg] = True
            # 如果当前有参数名，则存储参数值到字典中
            elif current_arg:
                self.actions[current_arg] = self._convert_value(
                    current_arg, arg)
                current_arg = None
            # 如果当前没有参数名，则直接存储参数到字典中
            else:
                self.actions[arg] = True

    def _convert_value(self, name, value):
        try:
            value = value.strip("'\"")
            if name in self.types:
                return self.types[name](value)
            return None
        except Exception:
            return None

    def usage(self):
        for name, info in self.args.items():
            if info.get('is_short') or info.get("is_def"):
                continue
            help_text = info['help'] or ''
            action_text = info['action'] or name
            short_text = info['short']
            if short_text is None:
                Terminal.print(f"    --{action_text}: {help_text}")
            else:
                Terminal.print((f"    --{action_text} | -{short_text}:"
                                f" {help_text}"))



# -*- coding: utf-8 -*-
from concurrent.futures import ThreadPoolExecutor, FIRST_COMPLETED, wait
import subprocess
import ffmpeg
import re
from tqdm import tqdm

from ddans.native.hook import NHook
from ddans.native.log import NLog
from ddans.native.system import NSystem
from ddans.pbar import TQDM_FORMAT

ValidFormats = ["mp4", "mkv", 'avi', "wmv", "rm", "ts", "mov", "mpeg", "rmvb"]

DefaultVcodec = "libx264"

Quality720 = {
    0: 2500,
    1: 2000,
    2: 1500,
}

Quality1080 = {
    0: 5000,
    1: 4000,
    2: 3000,
}

QualityAudio = {
    0: 320,
    1: 192,
    2: 128,
}


class NFfmpeg:

    @staticmethod
    def get_video_duration(input_file):
        if not NSystem.exists(input_file):
            NLog.error(f"{input_file} 不存在")
            return 0
        try:
            probe = ffmpeg.probe(input_file)
            return float(probe['format']['duration'])
        except Exception as e:
            NLog.error(f"get video duration {e}")
            return 0

    @staticmethod
    def get_video_resolution(url):
        try:
            # 调用 ffprobe 获取视频信息
            probe = NFfmpeg.get_probe(url)
            if not probe:
                return None

            # 查找视频流
            video_streams = [
                stream for stream in probe['streams']
                if stream['codec_type'] == 'video'
            ]
            if not video_streams:
                return None  # 没有视频流

            # 取第一个视频流的分辨率（通常是最主要的）
            stream = video_streams[0]
            width = int(stream['width'])
            height = int(stream['height'])
            return width, height

        # except ffmpeg.Error as e:
        #     NLog.error(f"get_video_resolution  {e.stderr.decode('utf-8')}")
        #     return None
        except Exception as e:
            NLog.error(f"get_video_resolution {e}")
            return None

    @staticmethod
    def get_probe(input_file):
        try:
            return ffmpeg.probe(input_file)
        except Exception as e:
            NLog.error(f"get_probe {e}")
            return None

    @staticmethod
    def compress_video(input_file,
                       output_file,
                       vcodec=DefaultVcodec,
                       acodec='aac',
                       video_bitrate='2500k',
                       resolution='1280x720'):
        try:
            if not NSystem.exists(input_file):
                NLog.error(f"{input_file} 不存在")
                return

            _, ext = NSystem.split_name_ext(input_file)
            if ext.lower() not in ValidFormats:
                NLog.error(f"视频压缩不支持该格式: {ext}")
                return

            NSystem.ensure_dir(output_file)

            # 设置输出参数
            (ffmpeg.input(input_file).output(
                output_file,
                vcodec=vcodec,
                acodec=acodec,
                video_bitrate=video_bitrate,
                vf=f'scale={resolution}').overwrite_output().run(quiet=True))
            NLog.success(f'compress {input_file} to {output_file}')
        except ffmpeg.Error as e:
            NLog.error(f'compress: {e.stderr.decode()}')
        except Exception as e:
            NLog.error(f'compress: {e}')
            pass

    @staticmethod
    def compress_video_ex(input_file,
                          output_file,
                          vcodec=DefaultVcodec,
                          acodec='aac',
                          resolution=720,
                          quality=0,
                          title="",
                          delete_file=False):

        try:
            if not NSystem.exists(input_file):
                NLog.error(f"{input_file} 不存在")
                return False

            name, ext = NSystem.split_name_ext(input_file)
            if not NSystem.isvalid_ext(input_file, ValidFormats):
                NLog.error(f"视频压缩不支持${ext}格式")
                return False

            probe = NFfmpeg.get_probe(input_file)
            if probe is None or 'format' not in probe:
                return False

            total_size = float(probe['format'].get('size', 0))
            total_duration = float(probe['format'].get('duration', 0))
            # total_size = NSystem.getsize(input_file)
            if total_size is None or total_size <= 0:
                return False

            # total_duration = NFfmpeg.get_video_duration(input_file)
            if total_duration is None or total_duration <= 0:
                return False

            # 查找视频流
            video_streams = [
                stream for stream in probe['streams']
                if stream['codec_type'] == 'video'
            ]
            if not video_streams:
                return None  # 没有视频流

            # 取第一个视频流的分辨率（通常是最主要的）
            stream = video_streams[0]
            width = int(stream.get('width', 0))
            height = int(stream.get('height', 0))

            min_size = min(width, height)
            if not min_size:
                return None

            NSystem.ensure_dir(output_file)
            is_gpu = vcodec != DefaultVcodec

            max_resolution = resolution
            # 分辨率小于等720，但设置1080时，调整成720
            if min_size <= 720 and resolution > 720:
                max_resolution = 720

            if max_resolution <= 720:
                bitrate = Quality720.get(quality, Quality720.get(0))
            else:
                bitrate = Quality1080.get(quality, Quality1080.get(0))

            video_bitrate = f'{bitrate}k'
            minrate = video_bitrate
            maxrate_num = int(
                min(int(re.match(r"\d+", minrate).group()) * 2, 10000))
            bufsize_num = int(maxrate_num * 2)
            cmd = [
                'ffmpeg', '-y', '-i', input_file, '-map', '0:v', '-map', '0:a',
                '-vcodec', vcodec, '-acodec', acodec
            ]
            if is_gpu:
                cmd.extend(['-b:v', video_bitrate, '-preset', 'veryslow'])
            else:
                # '-b:v', video_bitrate,
                # '-minrate': f'{minrate}'
                # '-crf', '23'
                # '-preset', 'slow',
                cmd.extend(['-b:v', video_bitrate, '-preset', 'slow'])
            cmd.extend([
                '-maxrate', f'{maxrate_num}k', '-bufsize', f'{bufsize_num}k',
                '-movflags', 'faststart', '-vf', f'scale=-1:{max_resolution}',
                output_file
            ])
            process = subprocess.Popen(cmd,
                                       bufsize=0,
                                       universal_newlines=True,
                                       shell=False,
                                       stderr=subprocess.PIPE,
                                       encoding="utf-8",
                                       text=True)

            total_size_mb = round(total_size / (1024 * 1024), 2)  # 转换为MB
            desc = title if NHook.isvalid_str(title) else name
            with tqdm(total=total_size_mb,
                      unit='MB',
                      unit_scale=False,
                      ncols=80,
                      bar_format=TQDM_FORMAT,
                      leave=False,
                      colour="cyan",
                      desc=f'{desc}') as pbar:

                while process.poll() is None:
                    try:
                        line = process.stderr.readline().strip()
                        if not NHook.isvalid_str(line):
                            continue
                        # duration_res = re.search(
                        #     r'Duration: (?P<duration>\S+)', line)
                        # if duration_res is not None:
                        #     duration = duration_res.groupdict()\
                        #                ['duration']
                        #     duration = re.sub(r',', '', duration)
                        result = re.search(r'time=(?P<time>\S+)', line)
                        if result is not None:
                            elapsed_time = result.groupdict()['time']

                            currentTime = NFfmpeg.get_seconds(elapsed_time)
                            estimated_size = (currentTime /
                                              total_duration) * total_size
                            estimated_mb = round(
                                estimated_size / (1024 * 1024), 2)
                            pbar.update(estimated_mb - pbar.n)
                        # result = re.search(r'size=\s*(?P<size>\d+)ki?b',
                        #                    line.lower())
                    # if result is not None:
                    #     estimated_size = float(result.groupdict()['size'])
                    #     estimated_mb = round(estimated_size / 1024, 2)
                    #     pbar.update(estimated_mb - pbar.n)
                    except Exception:
                        pass
            process.wait()
            if process.returncode != 0:
                # NLog.error('compress_video: failed')
                return False
            else:
                if delete_file:
                    NSystem.remove_file(input_file)
                # NLog.success(f'compress_video {input_file} to {output_file}')
                return True
        except ffmpeg.Error as e:
            NLog.error(f'compress: {e.stderr.decode()}')
            return False
        except Exception as e:
            NLog.error(f'compress: {e}')
            return False

    # @staticmethod
    # def get_seconds(time):
    #     h = int(time[0:2])
    #     # print("时：" + str(h))
    #     m = int(time[3:5])
    #     # print("分：" + str(m))
    #     s = int(time[6:8])
    #     # print("秒：" + str(s))
    #     ms = int(time[9:12])
    #     # print("毫秒：" + str(ms))
    #     ts = (h * 60 * 60) + (m * 60) + s + (ms / 1000)
    #     return ts
    @staticmethod
    def get_seconds(time_str):
        if ':' not in time_str:
            return 0
        """将时间字符串转换为秒数"""
        h, m, s = map(float, time_str.split(':'))
        return h * 3600 + m * 60 + s

    @staticmethod
    def ffmpeg_command(*args):
        """ 运行 FFmpeg 并返回输出 """
        try:
            cmd = ("ffmpeg", ) + args
            result = NSystem.run(*cmd)
            return result.stdout + result.stderr  # 可能信息在 stderr
        except FileNotFoundError:
            return ""  # FFmpeg 未安装

    @staticmethod
    def has_nvidia_gpu():
        """ 判断是否有 NVIDIA 显卡 """
        try:
            result = NSystem.run("nvidia-smi")
            # 如果返回结果包含 "NVIDIA-SMI" 或有显卡信息，说明有 NVIDIA 显卡
            return 'NVIDIA-SMI' in result.stdout
        except FileNotFoundError:
            # 如果没有安装 nvidia-smi 或没有 NVIDIA 显卡，会抛出 FileNotFoundError
            return False

    @staticmethod
    def get_available_encoders():
        """ 运行 FFmpeg 检测可用的 GPU 编码器 """
        output = NFfmpeg.ffmpeg_command("-encoders")
        encoders = {
            "nvenc":
            NSystem.is_windows and NFfmpeg.has_nvidia_gpu()
            and "hevc_nvenc" in output,
            "qsv":
            NSystem.is_windows and "hevc_qsv" in output,
            "vaapi":
            False,  # "h264_vaapi" in output,
            "videotoolbox":
            NSystem.is_darwin and "videotoolbox" in output
        }
        return encoders

    @staticmethod
    def select_hevc_encoder():
        available_encoders = NFfmpeg.get_available_encoders()
        """ 根据可用性选择最合适的 GPU 编码器 """
        if available_encoders["nvenc"]:
            return "hevc_nvenc"  # NVIDIA
        elif available_encoders["qsv"]:
            return "hevc_qsv"  # Intel QSV
        elif available_encoders["vaapi"]:
            return "hevc_amf"  # AMD VAAPI
        elif available_encoders["videotoolbox"]:
            return "hevc_videotoolbox"  # macOS
        else:
            return DefaultVcodec  # 纯软件编码（兼容模式）

    @staticmethod
    def select_encoder():
        available_encoders = NFfmpeg.get_available_encoders()
        """ 根据可用性选择最合适的 GPU 编码器 """
        if available_encoders["nvenc"]:
            return "h264_nvenc"  # NVIDIA
        elif available_encoders["qsv"]:
            return "h264_qsv"  # Intel QSV
        elif available_encoders["vaapi"]:
            return "h264_amf"  # AMD VAAPI
        elif available_encoders["videotoolbox"]:
            return "h264_videotoolbox"  # macOS
        else:
            return DefaultVcodec  # 纯软件编码（兼容模式）

    @staticmethod
    def compress_videos(dir="",
                        max=2,
                        resolution=720,
                        quality=0,
                        delete_file=False,
                        gpu=False):
        root = NSystem.get_full_path(dir)
        # print("compress_videos: %s" % root)
        if not NSystem.exists(root):
            NLog.error("compress directory not found!")
            return

        files = NSystem.listdir(root)
        arrs = list(
            filter(
                lambda i: NSystem.isfile(i) and NSystem.isvalid_ext(
                    i, ValidFormats), files))

        if len(arrs) <= 0:
            NLog.error("compress none")
            return

        futures = []
        vcodec = NFfmpeg.select_encoder() if gpu else DefaultVcodec
        with ThreadPoolExecutor(max_workers=max) as executor:
            for i in range(len(arrs)):
                file = arrs[i]
                old_name, ext = NSystem.split_name_ext(file)
                output_file = NSystem.join(root, 'out', f'F-{old_name}.mp4')
                futures.append(
                    executor.submit(NFfmpeg.compress_video_ex,
                                    file,
                                    output_file,
                                    delete_file=delete_file,
                                    quality=quality,
                                    vcodec=vcodec,
                                    resolution=resolution,
                                    title=str(i + 1)))

            pbar = tqdm(total=len(futures),
                        desc="compress",
                        unit='',
                        ncols=80,
                        bar_format=TQDM_FORMAT,
                        colour="red")
            while futures:
                done, futures = wait(futures, return_when=FIRST_COMPLETED)
                for future in done:
                    try:
                        future.result()  # 检查异常，但不阻塞
                    except Exception:
                        # NLog.error(f'视频压缩异常: {e}')
                        pass
                    pbar.update(1)
            pbar.close()

        NLog.success("compress: end")

    @staticmethod
    def export_audio(input_file,
                     output_file,
                     acodec='libmp3lame',
                     quality=1,
                     title="",
                     delete_file=False):

        try:
            if not NSystem.exists(input_file):
                NLog.error(f"{input_file} 不存在")
                return False

            name, ext = NSystem.split_name_ext(input_file)
            if not NSystem.isvalid_ext(input_file, ValidFormats):
                NLog.error(f"不支持${ext}格式")
                return False

            probe = NFfmpeg.get_probe(input_file)
            if probe is None or 'format' not in probe:
                return False

            total_size = float(probe['format'].get('size', 0))
            total_duration = float(probe['format'].get('duration', 0))
            # total_size = NSystem.getsize(input_file)
            if total_size is None or total_size <= 0:
                return False

            # total_duration = NFfmpeg.get_video_duration(input_file)
            if total_duration is None or total_duration <= 0:
                return False

            # 查找音频流
            # audio_streams = [
            #     stream for stream in probe['streams']
            #     if stream['codec_type'] == 'audio'
            # ]
            # if not audio_streams:
            #     return None  # 没有视频流

            # # 取第一个视频流的分辨率（通常是最主要的）
            # stream = audio_streams[0]
            # audio_bitrate = int(stream.get('bit_rate', 0))
            bitrate = QualityAudio.get(quality, QualityAudio.get(1, 0))

            min_bitrate = min(QualityAudio.get(0), bitrate)
            if not min_bitrate:
                return None

            NSystem.ensure_dir(output_file)

            # 设置音频比特率等参数
            cmd = [
                'ffmpeg', '-y', '-i', input_file, '-vn', '-acodec', acodec,
                '-b:a', f'{min_bitrate}k', '-vn', output_file
            ]
            process = subprocess.Popen(cmd,
                                       bufsize=0,
                                       universal_newlines=True,
                                       shell=False,
                                       stderr=subprocess.PIPE,
                                       encoding="utf-8",
                                       text=True)

            total_size_mb = round(total_size / (1024 * 1024), 2)  # 转换为MB
            desc = title if NHook.isvalid_str(title) else name
            with tqdm(total=total_size_mb,
                      unit='MB',
                      unit_scale=False,
                      ncols=80,
                      bar_format=TQDM_FORMAT,
                      leave=False,
                      colour="cyan",
                      desc=f'{desc}') as pbar:

                while process.poll() is None:
                    try:
                        line = process.stderr.readline().strip()
                        if not NHook.isvalid_str(line):
                            continue
                        # duration_res = re.search(
                        #     r'Duration: (?P<duration>\S+)', line)
                        # if duration_res is not None:
                        #     duration = duration_res.groupdict()\
                        #                ['duration']
                        #     duration = re.sub(r',', '', duration)
                        result = re.search(r'time=(?P<time>\S+)', line)
                        if result is not None:
                            elapsed_time = result.groupdict()['time']

                            currentTime = NFfmpeg.get_seconds(elapsed_time)
                            estimated_size = (currentTime /
                                              total_duration) * total_size
                            estimated_mb = round(
                                estimated_size / (1024 * 1024), 2)
                            pbar.update(estimated_mb - pbar.n)
                    except Exception:
                        pass
            process.wait()
            if process.returncode != 0:
                # NLog.error('compress_video: failed')
                return False
            else:
                if delete_file:
                    NSystem.remove_file(input_file)
                # NLog.success(f'compress_video {input_file} to {output_file}')
                return True
        except ffmpeg.Error as e:
            NLog.error(f'export: {e.stderr.decode()}')
            return False
        except Exception as e:
            NLog.error(f'export: {e}')
            return False

    @staticmethod
    def export_audios(dir="", max=2, quality=1):
        root = NSystem.get_full_path(dir)
        if not NSystem.exists(root):
            NLog.error("export directory not found!")
            return

        files = NSystem.listdir(root)
        arrs = list(
            filter(
                lambda i: NSystem.isfile(i) and NSystem.isvalid_ext(
                    i, ValidFormats), files))

        if len(arrs) <= 0:
            NLog.error("export none")
            return

        futures = []
        with ThreadPoolExecutor(max_workers=max) as executor:
            for i in range(len(arrs)):
                file = arrs[i]
                old_name, ext = NSystem.split_name_ext(file)
                output_file = NSystem.join(root, 'out', f'F-{old_name}.mp3')
                futures.append(
                    executor.submit(NFfmpeg.export_audio,
                                    file,
                                    output_file,
                                    quality=quality,
                                    title=str(i + 1)))

            pbar = tqdm(total=len(futures),
                        desc="export audios",
                        unit='',
                        ncols=80,
                        bar_format=TQDM_FORMAT,
                        colour="red")
            while futures:
                done, futures = wait(futures, return_when=FIRST_COMPLETED)
                for future in done:
                    try:
                        future.result()  # 检查异常，但不阻塞
                    except Exception:
                        # NLog.error(f'视频压缩异常: {e}')
                        pass
                    pbar.update(1)
            pbar.close()

        NLog.success("export_audios: end")



import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.header import Header
from email.utils import formataddr
from typing import List, Union

from ddans.native.hook import NHook
from ddans.native.log import NLog
from ddans.descriptor.singleton import SingletonMeta


class SEmailer(metaclass=SingletonMeta):

    def __init__(self, smtp_server, smtp_port, from_addr, password):
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port
        self.from_addr = from_addr
        self.password = password

    def send(self,
             to_addr: Union[str, List[str]],
             subject,
             body,
             from_addr='yury'):

        if isinstance(to_addr, str):
            to_addr = [to_addr]

        if not NHook.isvalid(to_addr):
            return

        # 创建一个MIMEMultipart对象，用于包含邮件的多个部分
        msg = MIMEMultipart()
        msg['From'] = formataddr((from_addr, self.from_addr), 'utf-8')  # 发送者名称
        msg['To'] = ", ".join(to_addr)
        msg['Subject'] = Header(subject, 'utf-8')  # 邮件主题

        # 邮件正文内容
        msg.attach(MIMEText(body, 'html', 'utf-8'))

        try:
            # 创建SMTP对象，并连接到SMTP服务器
            server = smtplib.SMTP_SSL(self.smtp_server, self.smtp_port)
            server.login(self.from_addr, self.password)  # 登录SMTP服务器
            server.sendmail(self.from_addr, to_addr, msg.as_string())  # 发送邮件
            server.quit()
            NLog.success("邮件发送成功")
            # NLog.success(f"{msg.as_string()}")
        except smtplib.SMTPException as e:
            NLog.error(f"邮件发送失败: {e}")



# -*- coding: utf-8 -*-
import json
import os
import shutil
import platform
import subprocess
from typing import List, Union

# from ddans.descriptor.static import StaticProperty
from ddans.domain.regex import Regex
from ddans.native.hook import NHook

root = os.getcwd()

ContentType = Union[str, bytes, dict, list]

system = platform.system()

isWindows = system == "Windows"
isLinux = system == "Linux"
isDarwin = system == "Darwin"

if isWindows:
    import winreg  # noqa: F401,E111

if isWindows:
    platformShort = "W"
elif isDarwin:
    platformShort = "M"
elif isLinux:
    platformShort = "L"
else:
    platformShort = ""


class NSystem():
    is_windows = isWindows
    is_linux = isLinux
    is_darwin = isDarwin
    platform_short = platformShort

    @staticmethod
    def get_root():
        return root

    @staticmethod
    def reset_root():
        return NSystem.set_current_path(root)

    @staticmethod
    def get_current_path():
        return os.getcwd()

    @staticmethod
    def set_current_path(path=''):
        os.chdir(path)

    @staticmethod
    def get_full_path(path, *args):
        return os.path.abspath(NSystem.join(path, *args))

    @staticmethod
    def join(path, *args):
        return os.path.join(path, *args)

    @staticmethod
    def dirname(path):
        return os.path.dirname(path)

    @staticmethod
    def exists(dirOrFile):
        return os.path.exists(dirOrFile)

    @staticmethod
    def get_dir(path):
        arr = os.path.split(path)
        dir = arr[0] or '.'
        file = arr[1]
        if len(file) <= 0:
            return dir
        elif "." not in file:
            return path
        return dir

    @staticmethod
    def ensure_dir(path="./", direct=False):
        dir = path if direct else NSystem.get_dir(path)
        _dir = NSystem.get_full_path(dir)
        if not os.path.exists(_dir):
            os.makedirs(_dir)

    @staticmethod
    def clean_dir(path):
        dir = NSystem.get_dir(path)
        if os.path.exists(dir):
            shutil.rmtree(dir)
            os.makedirs(dir)

    @staticmethod
    def split_name_ext(file: str):
        if len(file) <= 0:
            return "", ""
        tmp = os.path.splitext(file)
        name = os.path.basename(tmp[0])
        ext = tmp[1][1:].lower()
        return name, ext

    @staticmethod
    def isvalid_ext(name, ext: Union[str, List[str]]):
        if not NHook.isvalid_str(name):
            return False

        _, _ext = NSystem.split_name_ext(name)
        if not NHook.isvalid_str(_ext):
            return False

        exts = []
        if isinstance(ext, str):
            if NHook.isvalid_str(ext):
                exts = [ext]
            else:
                return False
        elif isinstance(ext, list):
            exts = ext
        else:
            return False

        lists = [
            item.lower() for item in exts if isinstance(item, str)
            and item.strip() != '' and exts.count(item) == 1
        ]

        count = len(lists)
        if count <= 0:
            return False
        return _ext.lower() in lists

    @staticmethod
    def listdir(path: str):
        try:
            names = os.listdir(path)
            return [NSystem.join(path, name) for name in names]
        except Exception:
            return []

    @staticmethod
    def basename(file: str):
        return os.path.basename(file)

    @staticmethod
    def get_directory_names(path):
        """
        获取路径名称列表

        倒叙名称，可能需要 reversed 反转
        """
        names = []
        try:
            while True:
                path, tail = os.path.split(path)
                if tail:
                    names.append(tail)
                else:
                    # 特殊处理Windows根目录，获取盘符的字母部分
                    if os.name == 'nt':
                        if path.endswith(':\\'):
                            names.append(path.replace(':\\', ''))
                        else:
                            names.append(path)
                    else:
                        # 特殊处理macOS和Linux根目录
                        if path == '/':
                            names.append('/')
                        else:
                            names.append(os.path.basename(path))
                    break
            # reversed(dirNames[:2])
            return names
        except Exception:
            return []

    @staticmethod
    def get_home():
        return os.path.expanduser('~')

    @staticmethod
    def get_sys_download():
        return os.path.expanduser('~/Downloads')

    @staticmethod
    def get_sys_document():
        return os.path.expanduser('~/Documents')

    @staticmethod
    def get_path():
        home = NSystem.get_home()
        dir = NSystem.get_full_path(home, ".ddan")
        NSystem.ensure_dir(dir, True)
        return dir

    @staticmethod
    def get_app_data():
        if isWindows:
            return os.path.expanduser('~/AppData/Roaming')
        elif isDarwin:
            return os.path.expanduser('~/Library/Application Support')
        elif isLinux:
            return os.path.expanduser('~/.config')
        else:
            return None

    @staticmethod
    def get_etc_path():
        if isWindows:
            return os.path.join(os.environ.get("SystemRoot", "C:\\Windows"),
                                "System32", "drivers", "etc")
        else:
            return "/etc"

    @staticmethod
    def open_folder(path):
        if not NSystem.exists(path):
            return False
        if isWindows:
            os.startfile(f"{path}")  # 在 Windows 上使用 startfile 函数打开文件夹
            return True
        elif isDarwin:
            os.system(f"open '{path}'")  # 在 macOS 上使用 open 命令打开文件夹
            return True
        elif isLinux:
            os.system(f"xdg-open '{path}'")  # 在 Linux 上使用 xdg-open 命令打开文件夹
            return True
        else:
            return False

    @staticmethod
    def system(cmd: str):
        if not NHook.isvalid_str(cmd):
            return
        os.system(cmd)

    @staticmethod
    def exec(*args):
        try:
            _args = NHook.to_list(args)
            result = subprocess.run(_args,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,
                                    text=True,
                                    check=True)
            return result.stdout.strip()
        except subprocess.CalledProcessError:
            # 捕捉到 git 命令错误时返回 None
            return None

    @staticmethod
    def run(*args):
        try:
            _args = NHook.to_list(args)
            return subprocess.run(_args,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
                                  text=True,
                                  check=True)
        except subprocess.CalledProcessError:
            # 捕捉到 git 命令错误时返回 None
            return None

    @staticmethod
    def check_output(*args):
        try:
            _args = NHook.to_list(args)
            result = subprocess.check_output(_args, text=True, shell=True)
            return result.strip()
        except subprocess.CalledProcessError as e:
            # 捕捉到 git 命令错误时返回 None
            print(f"Error occurred: {e}")
            return None
        except KeyboardInterrupt as e:
            print(f"Error occurred: {e}")
            return None
        except Exception as e:
            print(f"Error occurred: {e}")
            return None

    @staticmethod
    def shellex(cmd: str = ""):
        try:
            subprocess.run(cmd, text=True, check=True, shell=True)
            return True
        except subprocess.CalledProcessError:
            return False
        except KeyboardInterrupt:
            return False
        except Exception:
            return False

    @staticmethod
    def shell(cmd: str = "") -> bool:
        try:
            # 使用 Popen 启动子进程，并创建新的进程组
            process = subprocess.Popen(cmd, shell=True)
            process.wait()  # 等待子进程完成
            return process.returncode == 0  # 成功执行返回 True
        except subprocess.CalledProcessError:
            return False
        except KeyboardInterrupt:
            if process is not None and process.poll() is not None:
                process.kill()
                process.wait()
            return False
        except Exception:
            return False

    @staticmethod
    def open_terminal():
        current_path = os.getcwd()
        if isWindows:
            # Windows-specific command
            command = f'start cmd /K "cd /d {current_path}"'
        elif isLinux:
            # Linux-specific command (assuming gnome-terminal)
            command = (f'gnome-terminal -- bash -c "cd \\"{current_path}\\";'
                       f' exec bash"')
            # If using xterm, use the following command instead:
            # command = f'xterm -e "cd \\"{current_path}\\"; exec bash"'
        elif isDarwin:  # macOS is identified as "Darwin"
            # macOS-specific command using osascript
            script = f'''
            tell application "Terminal"
                do script "cd \\"{current_path}\\"; exec bash"
                activate
            end tell
            '''
            command = ["osascript", "-e", script]
        else:
            raise OSError(f"Unsupported operating system: {system}")

        # Execute the command
        if isDarwin:
            subprocess.Popen(command)
        else:
            subprocess.Popen(command, shell=True)

    @staticmethod
    def isdir(path):
        if not NHook.isvalid_str(path):
            return False
        return os.path.isdir(path)

    @staticmethod
    def isfile(path):
        if not NHook.isvalid_str(path):
            return False
        return os.path.isfile(path)

    @staticmethod
    def getsize(path):
        if not NSystem.exists(path):
            return 0
        return os.path.getsize(path)

    @staticmethod
    def read_data(file):
        try:
            res = None
            if not NSystem.isfile(file):
                return None
            with open(file, 'rb') as fs:
                # 从文件中读取数据
                res = fs.read()
            return res
        except Exception:
            return None

    @staticmethod
    def read_json(path, default=dict()):
        try:
            with open(path, 'r', encoding='utf8') as fp:
                map = json.load(fp)
            return map
        except IOError:
            # print('read_json ' + path + 'error', error)
            return default
        except Exception:
            return default
        # finally:
        # return map

    @staticmethod
    def write_json(dict_data, path, minify=False):
        try:
            NSystem.ensure_dir(path)
            with open(path, 'w', encoding='utf8', newline='') as fp:
                if minify:
                    fp.write(
                        json.dumps(dict_data,
                                   ensure_ascii=False,
                                   separators=(',', ':')))
                else:
                    fp.write(
                        json.dumps(dict_data, ensure_ascii=False, indent=2))
                return True
        except IOError as error:
            print('write_json ' + path + 'error', error)
            # print('')
            return False

    @staticmethod
    def write_file(buffer: ContentType, filename: str):
        if not NHook.isvalid_str(filename):
            return None
        if not NHook.isvalid(buffer, (str, bytes, dict, list)):
            return None
        _buffer = buffer
        if isinstance(buffer, str):
            _buffer = buffer.encode("utf8")
        elif isinstance(buffer, dict):
            content = json.dumps(buffer, ensure_ascii=False, indent=2)
            _buffer = content.encode("utf8")
        elif isinstance(buffer, list):
            content = str(buffer)
            _buffer = content.encode("utf8")
        else:
            return None

        fullname = NSystem.get_full_path(filename)
        NSystem.ensure_dir(fullname)
        with open(fullname, 'wb') as fs:
            fs.write(_buffer)
            fs.flush()
            fs.close()
        return fullname

    @staticmethod
    def copy(source: str, target: str):
        try:
            if not NHook.isvalid_str(source) or not NHook.isvalid_str(target):
                return None

            source_file = NSystem.get_full_path(source)
            target_path = NSystem.get_full_path(target)
            return shutil.copy2(source_file, target_path)
        except IOError:
            return None
        except Exception:
            return None

    @staticmethod
    def copy_file(source: str, target: str):
        try:
            if not NHook.isvalid_str(source) or not NHook.isvalid_str(target):
                return None

            source_file = NSystem.get_full_path(source)
            target_file = NSystem.get_full_path(target)
            NSystem.ensure_dir(target_file)
            return shutil.copyfile(source_file, target_file)
        except IOError:
            return None
        except Exception:
            return None

    @staticmethod
    def remove_file(filepath: str):
        try:
            if not NHook.isvalid_str(filepath):
                return None

            if not NSystem.exists(filepath):
                return None
            os.remove(filepath)
            return True
        except IOError:
            return None
        except Exception:
            return None

    @staticmethod
    def get_proxy():
        try:
            if isWindows:
                sub_key = ("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\"
                           "Internet Settings")
                registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,
                                              rf"{sub_key}")

                # 获取代理设置的值
                proxy_enable, _ = winreg.QueryValueEx(registry_key,
                                                      "ProxyEnable")
                proxy_server, _ = winreg.QueryValueEx(registry_key,
                                                      "ProxyServer")
                winreg.CloseKey(registry_key)
                if proxy_enable == 1 and NHook.isvalid_str(proxy_server):
                    return proxy_server
            elif isDarwin:
                # 执行 `scutil` 命令以获取代理设置
                result = NSystem.check_output(
                    'networksetup -getwebproxy "Wi-Fi"')
                output = result

                # 解析输出，提取代理设置
                proxies = {}
                for line in output.splitlines():
                    if line:
                        key, value = line.split(": ", 1)
                        proxies[key] = value

                proxy_enable = proxies.get("Enabled")
                proxy_server = proxies.get("Server")
                if proxy_enable == 'Yes' and NHook.isvalid_str(proxy_server):
                    return f"{proxy_server}:{proxies.get('Port', 0)}"
            elif isLinux:
                # 执行 `scutil` 命令以获取代理设置
                result = NSystem.check_output('env | grep https_proxy')
                if not NHook.isvalid_str(result):
                    return None
                output = result.strip()

                match = Regex.addrport.search(output)

                if match:
                    proxy_server = match.group(1)  # 第一个捕获组匹配 IP 地址
                    port = match.group(2)  # 第二个捕获组匹配端口号
                    return f"{proxy_server}:{port}"
            return None
        except Exception:
            return None



# -*- coding: utf-8 -*-
import math
import time
import requests
import urllib3
import concurrent.futures
from typing import Any, Literal, Mapping
from urllib import error
from tqdm import tqdm
from ddans.common.type import Callback
from ddans.native.hook import NHook
from ddans.native.system import NSystem
from ddans.native.url import NUrl
from ddans.native.log import NLog
from ddans.descriptor.singleton import singleton


@singleton
class SNetter(object):
    _http = None
    _timeout = urllib3.Timeout(total=30)

    def __init__(self):
        self._http = urllib3.PoolManager()
        self._executor = concurrent.futures.ThreadPoolExecutor(10)

    @property
    def http(self):
        return self._http

    def get_headers(self, headers: Mapping[str, str] | None = None):
        return {'Content-Type': 'application/json'} | (headers or {})

    def request(self,
                url: str,
                method: Literal['GET', 'POST'] = 'GET',
                params: Any | None = None,
                transform_body: Callback | None = None,
                headers: Mapping[str, str] | None = None):
        try:
            _url = url

            if method == 'GET':
                _url = '%s%s' % (url, NUrl.url_query(params, '?'))
                resp = self.http.request(method, url=_url, headers=headers)
            else:
                _headers = self.get_headers(headers)
                if transform_body is not None:
                    data = transform_body(params)
                else:
                    data = NHook.json_dumps(params)
                resp = self.http.request(method,
                                         url=_url,
                                         body=data,
                                         headers=_headers)
            if resp.status == 200:
                return resp.data.decode('utf-8') or ''
            else:
                return None
        except error.HTTPError as e:
            NLog.error("request HTTPError %s - %s" % (e.code, e.reason))
            return None
        except Exception as e:
            NLog.error("request error %s" % str(e))
            return None

    def download(self,
                 url: str,
                 dir: str = None,
                 name: str = None,
                 show_pbar=False) -> str | None:
        try:
            if not NHook.isvalid_str(url):
                return None

            _name = name or NSystem.basename(url)
            _dir = dir or NSystem.get_current_path()
            file = NSystem.join(_dir, _name)
            NSystem.ensure_dir(file)

            headers = {
                "Cache-Control": "no-cache"  # 告诉服务器不要缓存
            }

            params = {
                "t": int(time.time())  # 当前时间戳（秒级）
            }
            if show_pbar:
                resp = requests.get(url,
                                    stream=True,
                                    headers=headers,
                                    params=params)
                _total = float(resp.headers.get("Content-Length", 0))
                total = round(_total) / 1024 / 1024
                total = math.ceil(total * 100) / 100
                chunk_size = 1024 * 1024
                with open(file, 'wb') as f:
                    for data in tqdm(iterable=resp.iter_content(chunk_size),
                                     ncols=60,
                                     colour="cyan",
                                     desc="正在下载",
                                     total=total,
                                     unit='MB'):
                        f.write(data)
                return NSystem.get_full_path(file)
            else:
                resp = requests.get(url, headers=headers, params=params)
                with open(file, 'wb') as f:
                    f.write(resp.content)
                return NSystem.get_full_path(file)
        except error.HTTPError as e:
            NLog.error("download HTTPError %s - %s" % (e.code, e.reason))
            return None
        except Exception as e:
            NLog.error("download error %s" % str(e))
            return None

    def fetch(self,
              url: str,
              method: Literal['GET', 'POST'] = 'GET',
              params: Any | None = None,
              transform_data: Callback | None = None,
              headers: Mapping[str, str] | None = None):
        try:
            _url = url
            if method == 'GET':
                _url = '%s%s' % (url, NUrl.url_query(params, '?'))
                future = self._executor.submit(requests.get,
                                               _url,
                                               headers=headers)
            else:
                _headers = self.get_headers(headers)
                if transform_data is not None:
                    data = transform_data(params)
                else:
                    data = NHook.json_dumps(params)
                future = self._executor.submit(requests.post,
                                               _url,
                                               headers=_headers,
                                               data=data)
            resp = future.result()
            if resp.status_code == 200:
                return resp.content
            else:
                return None
        except error.HTTPError as e:
            NLog.error("fetch HTTPError %s - %s" % (e.code, e.reason))
            return None
        except Exception as e:
            NLog.error("fetch error %s" % str(e))
            return None



from setuptools import setup, find_packages

setup(
    name="ddan-py",
    version="0.1.10",
    packages=find_packages(),
    include_package_data=True,
    description="DDANs project library",
    long_description=open("README.md").read()
    if __name__ == "__main__" else "",
    long_description_content_type="text/markdown",
    author="yury",
    author_email="yry2580@qq.com",
    url="https://github.com/yry2580/ddan-tools",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=[
        "pygtrans", "requests", "urllib3", "tqdm", "ffmpeg-python"
    ],
    extras_require={
        "test": [
            "pytest",
        ],
    },
)



LANG_DICT = {
    "zh-CN": "zh-CN",
    "en": "en-US",
    "ru": "ru-RU",
    "vi": "vi-VN",
    "pt": "pt-BR",
    "es": "es-LA",
    "tr": "tr-TR",
    "th": "th-TH",
    "uk": "uk-UA",
    "hi": "hi-IN",
    "it": "it-IT",
    "id": "id-ID",
    "ja": "ja-JP",
    "fr": "fr-FR",
    "zh-TW": "zh-TW"
}

LANG_BASE_DICT = {
    "zh-CN": "简体中文",
    "en": "英文",
    "ru": "俄语",
    "vi": "越南语",
    "pt": "葡萄牙语",
    "es": "西班牙语",
    "tr": "土耳其语",
    "th": "泰语",
    "uk": "乌克兰语",
    "hi": "印地语",
    "it": "意大利语",
    "id": "印尼语",
    "ja": "日语",
    "fr": "法语",
    "zh-TW": "繁体中文"
}

LANG_DESC_DICT = {
    "zh-CN": "简体中文",
    "en": "英文 English",
    "ru": "俄语 Русский",
    "vi": "越南语 Tiếng Việt",
    "pt": "葡萄牙语 Português",
    "es": "西班牙语 Español",
    "tr": "土耳其语 Türkçe",
    "th": "泰语 แบบไทย",
    "uk": "乌克兰语 Yкраїнська",
    "hi": "印地语 हिंदी",
    "it": "意大利语 Italiano",
    "id": "印尼语 Bahasa Indonesia",
    "ja": "日语 日本語",
    "fr": "法语",
    "zh-TW": "繁体中文 繁體中文",
}

DEFAULT_LANG = 'zh-CN'



# -*- coding: utf-8 -*-
from typing import Callable
from ddans.common.event import DEvent
from ddans.descriptor.singleton import singleton
import threading
import time

Loop = "loop"


@singleton
class SWorker:

    def __init__(self):
        super()
        self._event = DEvent()
        self._running = threading.Event()
        self._stop_event = threading.Event()
        self._thread = threading.Thread(target=self._loop)
        self._thread.daemon = True  # 将线程设置为守护线程

    def start(self):
        if not self._thread.is_alive():
            self._running.set()
            self._stop_event.clear()
            self._thread = threading.Thread(target=self._loop)
            self._thread.daemon = True
            try:
                self._thread.start()
            except RuntimeError as e:
                print(f"线程启动失败: {e}")

    def stop(self):
        self._running.clear()
        self._stop_event.set()

    def _loop(self):
        while not self._stop_event.is_set():
            if self._running.is_set():
                try:
                    self._event.emit(Loop)
                    time.sleep(1)
                except Exception as e:
                    print(f"在循环中捕获异常: {e}")
                    self._running.clear()

    def onLoop(self, callback: Callable[[], None]):
        try:
            self._event.on(Loop, callback)
        except Exception as e:
            print(f"注册回调函数失败: {e}")

    def offLoop(self, callback: Callable[[], None]):
        try:
            self._event.off(Loop, callback)
        except Exception as e:
            print(f"取消注册回调函数失败: {e}")



# -*- coding: utf-8 -*-
import mammoth
import markdown
from ddans.native.hook import NHook
from ddans.native.system import NSystem


class FConvert():
    @staticmethod
    def read_md_from_doc(file: str, default=None):
        try:
            if not NHook.isvalid_str(file):
                return default
            filepath = NSystem.get_full_path(file)
            if not NSystem.exists(filepath):
                return default
            result = mammoth.convert_to_markdown(filepath)
            return result.value
        except Exception:
            return default

    @staticmethod
    def read_html_from_doc(file: str, default=None):
        try:
            if not NHook.isvalid_str(file):
                return default
            filepath = NSystem.get_full_path(file)
            if not NSystem.exists(filepath):
                return default
            md = mammoth.convert_to_markdown(filepath)
            return FConvert.markdown_to_html(md.value, default)
        except Exception:
            return default

    @staticmethod
    def read_html_from_md(file: str, default=None):
        try:
            if not NHook.isvalid_str(file):
                return default
            filepath = NSystem.get_full_path(file)
            if not NSystem.exists(filepath):
                return default
            text = NSystem.read_data(filepath)
            return markdown.markdown(NHook.buffer_utf8(text))
        except Exception:
            return default

    @staticmethod
    def markdown_to_html(content: str, default=None):
        try:
            if not NHook.isvalid_str(content):
                return default
            return markdown.markdown(content)
        except Exception:
            return default
